package coloreti;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.io.IOException;

import java.util.*;

import javax.swing.JOptionPane;

import coloreti.IOEvent.IOEventType;
import coloreti.data.*;
import coloreti.gui.*;
import coloreti.interfaces.*;
import coloreti.network.client.ClientBase;
import coloreti.network.exceptions.SevereException;
import coloreti.network.messages.*;
import coloreti.network.messages.GameActionMessage.GameActionType;
import coloreti.network.messages.InitialMessage.InitialMessageType;

public class GameController implements MouseController, KeyListener {
	
	/* Constants go here */
	private static final int SCREEN_CONNECT = 1;
	private static final int SCREEN_LOBBY = 2;
	private static final int SCREEN_GAME = 3;
	
	private static final int BUTTON_CONNECT = 0;
	private static final int BUTTON_READY = 1;
	private static final int BUTTON_READY_PRESSED = 2;
	
	private static final long GEM_GENERATION_COOLDOWN = 500;
	private static final int GEM_MINIMUM_THRESHOLD = 10;
	
	/* State variables here */
	private GuiBase base;
	private ClientBase client;
	private int currentScreen;
	private GameScreen gameScreen;
	private Lobby lobby;
	private Board board;
	
	private int currentPlayer;
	private long currentTime;
	private long remainingTime;
	private int[][] scores;
	private int[] positions;
	
	//indicates that a game is over. triggers when it receives a game over message
	private boolean gameOver;
	
	private PriorityQueue<IOEvent> ioEventQueue;
	
	private LinkedList<Message> incomingMessages;
	private LinkedList<Message> outgoingMessages;
	
	public GameController(GuiBase base)
	{
		this.base = base;
		currentScreen = SCREEN_LOBBY;
		lobby = new Lobby(this, 800, 600);
		client = new ClientBase(this);
		
		currentPlayer = 0;
		currentTime = 0;
		remainingTime = -1;
		
		ioEventQueue = new PriorityQueue<IOEvent>();
		incomingMessages = new LinkedList<Message>();
		outgoingMessages = new LinkedList<Message>();

	}
	
	
	/* General Purpose Functions */
	private void exitGame()
	{
		System.exit(0);
	}
	
	/* IO Call functions */
	private void queueEvent(IOEventType type)
	{
		synchronized (ioEventQueue)
		{
			IOEvent event = new IOEvent(type, currentPlayer, currentTime);
			ioEventQueue.offer(event);
		}
		
	}
	
	private void queueGameAction(GameActionType type)
	{
		GameActionMessage gam = new GameActionMessage(currentPlayer, type);
		
		if(type.equals(type.ACTIVATE_GEMS))
		{
			int bitstream = (board.getPlayers().get(currentPlayer)).bitStreamActiveGems();
			gam.setData(bitstream);
		}
		
		synchronized (outgoingMessages)
		{
			outgoingMessages.offer(gam);
		}
	}
	
	private void drawGameOverScreen(Graphics2D g, int x, int y)
	{
		g.setColor(new Color(192, 192, 192));
		g.fillRect(70, 60, 660, 480);
		g.setColor(Color.black);
		g.drawRect(70, 60, 660, 480);
		int numPlayers = board.getPlayers().size();
		int partition = 600 / numPlayers;
		g.drawLine(70, 90, 730, 90);
		for (int i = 0; i < Gem.NUM_GEMS; i++)
		{
			g.drawImage(Gem.getGemImage(i), 90, 95 + (Gem.GEM_RADIUS * 2 + 10) * i, null);
		}
		GameScreen.drawCenteredString(g, 100, 110 + (Gem.GEM_RADIUS * 2 + 10) * (Gem.NUM_GEMS), "Place");
		
		g.setFont(new Font("Verdana", Font.BOLD, 14));
		g.setColor(new Color(0, 0, 0));
		for (int i = 0; i < numPlayers; i++)
		{
			Player p = board.getPlayers().get(i);
			g.drawLine(130 + partition * i, 60, 130 + partition * i, 540);
			int centerx = 130 + partition * i + partition / 2;
			
			GameScreen.drawCenteredString(g, centerx, 85, p.getName());
			for(int j = 0; j < p.getLevels().length; j++)
			{
				GameScreen.drawCenteredString(g, centerx, 110 + (Gem.GEM_RADIUS * 2 + 10) * j, p.getLevels()[j]+"");
			}
			
			GameScreen.drawCenteredString(g, centerx, 110 + (Gem.GEM_RADIUS * 2 + 10) * (Gem.NUM_GEMS), positions[i] + "");
		}
	}
	
	@Override
	public void drawComponent(Graphics2D g, int x, int y)
	{
		switch (currentScreen)
		{
		case SCREEN_LOBBY:
			lobby.drawComponent(g,x,y);
			break;
		case SCREEN_GAME:
			gameScreen.drawComponent(g, x, y);
			if (gameOver)
			{
				drawGameOverScreen(g, x, y);
			}
			break;
		}
		
		
	}

	@Override
	public void update(long elapsedTime) 
	{
		switch (currentScreen)
		{
		case SCREEN_LOBBY:
			updateLobbyScreen(elapsedTime);
			break;
		case SCREEN_GAME:
			updateGameScreen(elapsedTime);
		}
	}
	
	/* Update functions */
	private void updateLobbyScreen(long elapsedTime)
	{
		try
		{
			while (!incomingMessages.isEmpty())
			{
				Message msg = incomingMessages.poll();
				if (msg instanceof TextMessage)
				{
					lobby.getChatLines().add(((TextMessage)msg).getText());
				}
				else if (msg instanceof InitialMessage)
				{
					InitialMessage imsg = (InitialMessage)msg;
					switch (imsg.getType())
					{
					case CLIENT_JOINED:
						lobby.getListOfPlayers().add(imsg.getData());
						lobby.getChatLines().add(imsg.getData() + " has entered the lobby.");
						break;
					case CLIENT_LEFT:
						lobby.getListOfPlayers().remove(imsg.getData());
						lobby.getChatLines().add(imsg.getData() + " has left the lobby.");
						break;
					default:
						break;
					}
				}
				else if (msg instanceof GameStartMessage)
				{
					// woohoo, we're ready to start the game
					GameStartMessage gsm = (GameStartMessage)msg;
					board = new Board(gsm.getWidth(), gsm.getHeight());
					remainingTime = gsm.getTotalTime();
					board.getGems().addAll(gsm.getGemData());
					board.getBoundaries().addAll(gsm.getBoundaries());
					for (int i = 0; i < gsm.getPlayerData().size(); i++)
					{
						Player player = PlayerInitData.createNewPlayer(gsm.getPlayerData().get(i), 
								board);
						board.getPlayers().add(player);	
					}
					currentPlayer = gsm.getReceiverId();
					client.setId(currentPlayer);
					gameScreen = new GameScreen(board, 800, 600, this);
					gameScreen.setCurrentPlayer(currentPlayer);
					currentScreen = SCREEN_GAME;
				}
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			System.exit(0);
		}
		
		lobby.update(elapsedTime);
	}
	
	private void updateGameScreen(long elapsedTime)
	{
		//if game is over, dont update
		if(gameOver)
		{
			return;
		}
		
		currentTime += elapsedTime;
		sendOutgoingMessages();
		receiveIncomingMessages();
		
		for (int i = 0; i < board.getProjectiles().size(); i++)
		{
			Projectile proj = board.getProjectiles().get(i);
			proj.update(elapsedTime);
			if ((proj.getDuration() == Projectile.INFINITE_DURATION &&
					board.willCollideWithTerrain(proj.getX(), proj.getY(), proj.getRadius()))
					|| proj.getDuration() <= 0)
			{
				board.getProjectiles().remove(i);
				i--;
			}
		}
		
		for (Player player : board.getPlayers())
		{
			// update the player's motion
			player.update(elapsedTime);
			
			int projIndex = board.collisionWithProjectiles(player);
			// check if a projectile hit the player
			if (projIndex != -1)
			{
				Projectile proj = board.getProjectiles().get(projIndex);
				if (proj.getIndex() == 6 && player.getStatus() != Player.INVINCIBLE_STATUS)
				{
					// player got hit by a flying bicep
					player.applyAcceleration(proj.getDx() * (3 + proj.getLevel()), 
							proj.getDy() * (3 + proj.getLevel()));
				}
				else if (player.getStatus() != proj.getIndex() && 
						player.getStatus() != Player.INVINCIBLE_STATUS)
				{
					player.setStatus(proj.getIndex(), proj.getEffectDuration());
				}
				
				if (proj.getDuration() == Projectile.INFINITE_DURATION)
				{
					proj = board.getProjectiles().remove(projIndex);
				}
			}
		}
	}
	
	/* Callback functions */
	
	/**
	 * This callback is called whenever the user presses a button in the lobby
	 * @param currButtonState The current state of the button
	 */
	public void lobbyButtonPressed(int currButtonState)
	{
		switch (currButtonState)
		{
		case Lobby.BUTTON_CONNECT:
			// connect to the server
			String playerName = "", hostName = "", portName = "";
			int portNum = -1;
			while (hostName.equals(""))
			{
				hostName = JOptionPane.showInputDialog(base, "Enter host name");
				if (hostName == null)
				{
					System.exit(0);
				}
				hostName = hostName.trim();
			}
			
			while (portNum == -1)
			{
				portName = JOptionPane.showInputDialog(base, "Enter port number");
				try
				{
					portNum = Integer.parseInt(portName);
				} 
				catch (NumberFormatException e)
				{
				}
				if (portNum <= 0 || portNum > 65535)
				{
					portNum = -1;
					JOptionPane.showMessageDialog(base, "Port number must be a number between 1 and 65535");
				}
			}
			
			boolean connected = client.connect(hostName, portNum);
			if (!connected)
			{
				JOptionPane.showMessageDialog(base, "Failed to initialize connection to " + hostName);
				System.exit(0);
			}
			
			while (playerName.equals(""))
			{
				playerName = JOptionPane.showInputDialog(base, "Enter your username");
				playerName = playerName.trim();
				
				if (playerName.length() > 12)
				{
					JOptionPane.showMessageDialog(base, "Player name may not be longer than 12 characters.");
					playerName = "";
				}
			}
			
			try
			{
				// First, send player's name to the server
				client.sendInitialMessage(playerName, InitialMessageType.NAME_SEND);
				InitialMessage welcomeMsg = null;
				while (welcomeMsg == null)
				{
					welcomeMsg = client.readMessage(InitialMessage.class);
				}
				// receive Welcome packet containing list of players
				if (!welcomeMsg.getType().equals(InitialMessageType.WELCOME))
				{
					throw new SevereException("Unexpected message received from server.");
				}
				String nameList = welcomeMsg.getData();
				StringTokenizer st = new StringTokenizer(nameList, ";");
				while (st.hasMoreTokens())
				{
					lobby.getListOfPlayers().add(st.nextToken());
				}
				lobby.setPlayerName(playerName);
				lobby.setConnected(true);
				lobby.getChatLines().add(playerName + " has entered the lobby.");
				lobby.setButtonState(Lobby.BUTTON_READY);
				// start client to automatically receive messages
				Thread thread = new Thread(client);
				thread.start();
			}
			catch (Exception e)
			{
				e.printStackTrace();
				System.exit(0);
			}
			break;
		case Lobby.BUTTON_READY:
			try
			{
				client.sendInitialMessage("", InitialMessageType.READY);
				lobby.setButtonState(Lobby.BUTTON_READY_PRESSED);
			}
			catch (Exception e)
			{
				e.printStackTrace();
				System.exit(0);
			}
			
			break;
		case Lobby.BUTTON_READY_PRESSED:
			break;
		}
	}
	
	/**
	 * This method sends a text message to the server if connected
	 * @param message The message to send
	 */
	public void sendTextMessage(String message)
	{
		if (!client.isConnected())
		{
			return;
		}
		try 
		{
			client.sendTextMessage(message);
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * This method sends a message to the server if connected
	 * @param message The message to send
	 */
	public void sendMessage(Message message)
	{
		if (!client.isConnected())
		{
			return;
		}
		try
		{
			client.sendMessage(message);
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	
	private void sendOutgoingMessages()
	{
		synchronized (outgoingMessages)
		{
			while (!outgoingMessages.isEmpty())
			{
				sendMessage(outgoingMessages.poll());
			}
		}
	}
	
	/**
	 * This method dequeues the IO Events and applies them
	 */
	private void receiveIncomingMessages()
	{
		
		synchronized(incomingMessages)
		{
			while (!incomingMessages.isEmpty())
			{	
				Message msg = incomingMessages.poll();
				if(msg instanceof GameOverMessage)
				{
					GameOverMessage gom = (GameOverMessage)msg;
					scores = gom.getScores();
					positions = getPositions(scores);
					gameOver = true;
				}
				else if (msg instanceof PlayerStateMessage)
				{
					PlayerStateMessage psm = (PlayerStateMessage)msg;
					remainingTime = psm.getRemainingTime();
					for (int i = 0; i < psm.getPlayerData().size(); i++)
					{
						Player p = board.getPlayers().get(i);
						p.applyUpdate(psm.getPlayerData().get(i));
					}
				}
				else if(msg instanceof GameInfoMessage)
				{
					GameInfoMessage gim = (GameInfoMessage)msg;
					switch(gim.getType()){
					case GEM_CREATED:
						Gem gem = new Gem(gim.getX(), gim.getY(), gim.getData1());
						board.getGems().add(gem);
						break;
						
					case GEM_PICKED_UP:
						int playerIndex = gim.getData1();
						int gemIndex = gim.getData2();
						int gemColor = gim.getData3();
						System.out.println("Player " + playerIndex + " received gem pick up message");
						
						float gemPosX = gim.getX();
						float gemPosY = gim.getY();
						ArrayList<Gem> gems = board.getGems();
						
						//add to player
						Player p = board.getPlayers().get(playerIndex);
						p.gainedGem(gemColor);
						
						// find the gem corresponding to that data and remove it
						boolean found = false;
						for (int i = 0; i < gems.size(); i++)
						{
							gem = gems.get(i);
							if (gem.getColor() == gemColor && Math.abs(gemPosX - gem.getX()) < 0.001f
									&& Math.abs(gemPosY - gem.getY()) < 0.001f)
							{
								found = true;
								gems.remove(i);
								break;
							}
						}
						
						if (!found) 
						{
							System.out.println("Desynchronization problem: gem not found.");
						}
						
						break;
						
					case ATTACK_FIRED:
						System.out.println("attack was fired");
						int pIndex = gim.getData1();
						int data = gim.getData2();
						int[] gemArr = splitBitStream(data);
						int[] attack = parseAttackData(gemArr);
						Player player = board.getPlayers().get(pIndex);						
						player.removeGems(gemArr);
						
						// create a projectile
						if (attack[1] == 3)
						{
							// the player leveled up in a color
							player.setStatus(Player.CAST_STATUS, 1000);
							player.getLevels()[attack[0]]++;
						}
						else if (attack[1] == 2)
						{
							// the player fired a major attack
							int level = player.getLevels()[attack[0]];
							
							if (attack[0] == 2)
							{
								player.setStatus(Player.HASTE_STATUS, 
										(int)(Projectile.EFFECT_DURATION[attack[0]] * (1 + level * 0.2f)));
							}
							else if (attack[0] == 4)
							{
								player.setStatus(Player.INVINCIBLE_STATUS,
										(int)(Projectile.EFFECT_DURATION[attack[0]] * (1 + level * 0.2f)));
							}
							else if (attack[0] == 5)
							{
								// someone got struck by lightning
								double mindistsq = Double.MAX_VALUE;
								int minindex = -1;
								for (int i = 0; i < board.getPlayers().size(); i++)
								{
									p = board.getPlayers().get(i);
									if (i == player.getIndex()) 
									{
										continue;
									}
									double dist = (p.getX() - player.getX()) * (p.getX() - player.getX())
										+ (p.getY() - player.getY()) * (p.getY() - player.getY());
									
									if (minindex == -1 || dist < mindistsq)
									{
										minindex = i;
										mindistsq = dist;
									}
								}
								
								board.getPlayers().get(minindex).setStatus(Player.LIGHTNING_STATUS, 
										(int)(Projectile.EFFECT_DURATION[attack[0]] * (1 + level * 0.2f)));
							}
							else
							{
								int radius = Projectile.BASE_RADIUS[attack[0]] + 
									level * Projectile.INCREMENT_RADIUS[attack[0]];
								Projectile proj = new Projectile (gim.getX(), gim.getY(), radius, attack[0]);
								proj.setLevel(level);
								proj.setDx(Projectile.INITIAL_VELOCITY[attack[0]] * (float)Math.cos(gim.getFacing()));
								proj.setDy(Projectile.INITIAL_VELOCITY[attack[0]] * (float)Math.sin(gim.getFacing()));
								proj.setOriginator(player.getIndex());
								proj.setDuration(Projectile.PROJECTILE_DURATION[attack[0]]);
								board.getProjectiles().add(proj);
							}
						}
						break;
					}
				}
			}
		}
		
		synchronized(ioEventQueue)
		{
			while (!ioEventQueue.isEmpty())
			{	
				if (ioEventQueue.peek().getTime() > currentTime)
				{
					/* This may happen in a multiplayer scenario where
					 * some computers may be "ahead" due to latency issues.
					 * Refuse to process any events that are ahead of the current
					 * time
					 */
					break;
				}
				
				IOEvent event = ioEventQueue.poll();
				Player player = board.getPlayers().get(event.getPlayer());
				switch (event.getType())
				{
				case MOVING_FORWARD:
					player.setMotion(Player.MOTION_FORWARD);
					break;
				case MOVING_BACKWARD:
					player.setMotion(Player.MOTION_BACKWARD);
					break;
				case STOP_MOVING:
					player.setMotion(Player.MOTION_NONE);
					break;
				case TURNING_LEFT:
					player.setTurning(Player.TURNING_LEFT);
					break;
				case TURNING_RIGHT:
					player.setTurning(Player.TURNING_RIGHT);
					break;
				case STOP_TURNING:
					player.setTurning(Player.TURNING_NONE);
					break;
				case ACTIVATE_GEMS:
					
				}
				
			}
		}
	
	}
	
	public void messageReceived(Message m)
	{
		synchronized (incomingMessages)
		{
			incomingMessages.offer(m);
		}
	}
	
	@Override
	public void mouseDragged(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void mouseClicked(MouseEvent e) {
		
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent e) 
	{
		int x = e.getX()-2;
		int y = e.getY()-31;
		//values gotten from lobby
		
		switch (currentScreen)
		{
		case SCREEN_LOBBY:
			if (e.getButton() == MouseEvent.BUTTON1)
			{
				lobby.leftMouseClicked(x, y);
			}
		case SCREEN_GAME:
			if (true) break;
			if (e.getButton() == MouseEvent.BUTTON1)
			{
				gameScreen.mouseQuery(x, y);
			}
			else
			{
				System.out.println("Break");
			}
		}
		
	}

	@Override
	public void mouseReleased(MouseEvent e) 
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
		{
			exitGame();
		}
		
		if (currentScreen == SCREEN_GAME)
		{
			switch (e.getKeyCode())
			{
			case KeyEvent.VK_W:
				queueGameAction(GameActionType.MOVING_FORWARD);
				//queueEvent(IOEventType.MOVING_FORWARD);
				break;
			case KeyEvent.VK_A:
				queueGameAction(GameActionType.TURNING_LEFT);
				//queueEvent(IOEventType.TURNING_LEFT);
				break;
			case KeyEvent.VK_D:
				queueGameAction(GameActionType.TURNING_RIGHT);
				//queueEvent(IOEventType.TURNING_RIGHT);
				break;
			case KeyEvent.VK_S:
				queueGameAction(GameActionType.MOVING_BACKWARD);
				//queueEvent(IOEventType.MOVING_BACKWARD);
				break;
			case KeyEvent.VK_SPACE:
				//check if player has 3 gems
				Player p = board.getPlayers().get(currentPlayer);
				//only send if the number of valid gems in activate slot is 3
				System.out.println("numgems: " + p.numGems());
				if(p.numGems()==3)
				{
					queueGameAction(GameActionType.ACTIVATE_GEMS);
				}
				break;
			case KeyEvent.VK_Z:
				gameScreen.swapGems();
				break;
			case KeyEvent.VK_1:
				gameScreen.selectGem(1);
				break;
			case KeyEvent.VK_2:
				gameScreen.selectGem(2);
				break;
			case KeyEvent.VK_3:
				gameScreen.selectGem(3);
				break;
			case KeyEvent.VK_4:
				gameScreen.selectGem(4);
				break;
			case KeyEvent.VK_5:
				gameScreen.selectGem(5);
				break;
			}
		}
		
		if(currentScreen == SCREEN_LOBBY){
			if(lobby.isChat()){
				lobby.keyPressed(e);
			}
		}
	}

	@Override
	public void keyReleased(KeyEvent e) {
		
		if (currentScreen == SCREEN_GAME)
		{
			switch(e.getKeyCode())
			{
			case KeyEvent.VK_W:
			case KeyEvent.VK_S:
				queueGameAction(GameActionType.STOP_MOVING);
				//queueEvent(IOEventType.STOP_MOVING);
				break;
			case KeyEvent.VK_A:
			case KeyEvent.VK_D:
				queueGameAction(GameActionType.STOP_TURNING);
				//queueEvent(IOEventType.STOP_TURNING);
				break;
			}
		}
		
	}
	@Override
	public void keyTyped(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	/* Helper methods */
	
	public static int[] splitBitStream(int data)
	{
		int color1 = (data & (0x7 << 6)) >> 6;
		int color2 = (data & (0x7 << 3)) >> 3;
		int color3 = data & 0x7;
		return new int[] {color1, color2, color3};
	}
	
	/**
	 * This method parses the attack data given as an array of int. It returns
	 * an array of integers specifying: 
	 * a) the color of the attack
	 * b) the type of the attack, where 3 represents a 3-gem level up, 2
	 * represents a 2-gem attack, and 1 represents a weak multicolor attack 
	 */
	public static int[] parseAttackData(int[] data)
	{
		if (data[0] == data[1])
		{
			if (data[0] == data[2])
			{
				return new int[] {data[0], 3};
			}
			else
			{
				return new int[] {data[0], 2};
			}
		}
		else if (data[0] == data[2])
		{
			return new int[] {data[0], 2};
		}
		else if (data[1] == data[2])
		{
			return new int[] {data[1], 2};
		}
		return new int[] {data[0], 1};
	}
	
	public long getRemainingTime() {
		return remainingTime;
	}


	public void setRemainingTime(long remainingTime) {
		this.remainingTime = remainingTime;
	}
	
	//TODO:: HANDLE TIES
	public int determineWinner(int[][] scores)
	{
		int winner = -1;
		int[] playerMin = new int[6];
		
		for(int i=0; i < scores.length; i++){
			//a players minimum
			int minimum = scores[i][0];
			for(int j=0; j<scores[i].length;j++){
				//if the current minimum is greater than current integer
				if((scores[i][j])<minimum){
					minimum = scores[i][j];
				}
			}
			
			playerMin[i] = minimum;
			
		}
		
		winner = 0;
		for(int i=0; i<playerMin.length;i++){
			if(playerMin[i]<playerMin[winner]){
				winner = i;
			}
		}
		
		return winner;
		
	}
	
	/**
	 * Get the positions of all players
	 * @param scores The raw scores of the players
	 * @return The positions of the players
	 */
	public int[] getPositions(int[][] scores)
	{
		int[] positions = new int[scores.length];
		
		for (int i = 0; i < scores.length; i++)
		{
			int numberLost = 1;
			for (int j = 0; j < scores.length; j++)
			{
				if (i == j)
				{
					continue;
				}
				int result = compareScores(scores[i], scores[j]);
				if (result == -1)
				{
					numberLost++;
				}
			}
			positions[i] = numberLost;
		}
		return positions;
	}
	
	public int compareScores(int[] score1, int[] score2)
	{
		int[] tempArr1 = new int[score1.length];
		int[] tempArr2 = new int[score2.length];
		System.arraycopy(score1, 0, tempArr1, 0, score1.length);
		System.arraycopy(score2, 0, tempArr2, 0, score2.length);
		
		Arrays.sort(tempArr1);
		Arrays.sort(tempArr2);
		for (int i = 0; i < tempArr1.length; i++)
		{
			if (tempArr1[i] > tempArr2[i])
			{
				return 1;
			}
			else if (tempArr1[i] < tempArr2[i])
			{
				return -1;
			}
		}
		
		return 0;
	}
}
