package coloreti;

import java.io.IOException;
import java.util.*;
import java.util.logging.Logger;

import coloreti.data.*;
import coloreti.network.messages.GameActionMessage;
import coloreti.network.messages.GameInfoMessage;
import coloreti.network.messages.GameOverMessage;
import coloreti.network.messages.GameStartMessage;
import coloreti.network.messages.PlayerStateMessage;
import coloreti.network.messages.GameInfoMessage.GameInfoType;
import coloreti.network.server.ServerBase;


/**
 * This class represents the engine behind the game. It contains the game
 * states as well as update and verification methods to ensure data is
 * consistent.
 * @author ctheng
 * @author bsee
 */
public class GameEngine {
	
	public final static int NUM_START_GEMS = 14;
	public final static int DEFAULT_TIME_LIMIT = 600000;
	public final static int GEM_CREATION_DELAY = 1000;
	public final static int UPDATE_INTERVAL = 60;
	public final static int[] PLAYER_START_X = {339, 1704, 2950, 2354, 786, 373 };
	public final static int[] PLAYER_START_Y = {186, 155, 748, 2092, 2126, 1071 };
	
	private ServerBase server;
	private Board board;
	private Random rng;
	private int remainingTime;
	private int timeToSendUpdates;
	private int gemCreationTime;
	
	//indicates that game is over
	private boolean gameOver;
	
	Logger LOGGER = Logger.getLogger("tta.server.ServerBase");
	
	public GameEngine(ServerBase server)
	{
		this.server = server;
		rng = new Random();
		remainingTime = -1;
		timeToSendUpdates = 0;
		gemCreationTime = 0;
		gameOver = false;
	}
	
	public void generateNewGame(int numPlayers)
	{
		board = new Board(3200, 2400);
		board.loadBoundaries("textures/collision.txt");
		for (int i = 0; i < numPlayers; i++)
		{
			Player player = new Player(PLAYER_START_X[i], PLAYER_START_Y[i], board, i);
			board.getPlayers().add(player);
		}
		
		while (board.getGems().size() < NUM_START_GEMS)
		{
			float x = rng.nextFloat() * board.getWidth();
			float y = rng.nextFloat() * board.getHeight();
			if (board.willCollide(x, y, Gem.GEM_RADIUS * 2))
			{
				continue;
			}
			if (board.willCollideWithPlayers(x, y, Player.PLAYER_RADIUS * 2, -1))
			{
				continue;
			}
			Gem gem = new Gem(x, y, rng.nextInt(Gem.NUM_GEMS));
			board.getGems().add(gem);
			
		}
	}
	
	public GameStartMessage getGameStartMessage(int senderId)
	{
		GameStartMessage gsm = new GameStartMessage(senderId, -1);
		for (int i = 0; i < board.getPlayers().size(); i++)
		{
			Player p = board.getPlayers().get(i);
			gsm.getPlayerData().add(p.getPlayerInitData());
		}
		
		for (int i = 0; i < board.getGems().size(); i++)
		{
			gsm.getGemData().add(board.getGems().get(i));
		}
		
		gsm.getBoundaries().addAll(board.getBoundaries());
		gsm.setWidth(board.getWidth());
		gsm.setHeight(board.getHeight());
		gsm.setTotalTime(DEFAULT_TIME_LIMIT);
		return gsm;
	}
	
	/**
	 * Gets the board used for this game
	 * @return The board in the game
	 */
	public Board getBoard()
	{
		return board;
	}
	
	/**
	 * Starts the game
	 */
	public void startGame()
	{
		remainingTime = DEFAULT_TIME_LIMIT;
	}
	
	/**
	 * Applies an action to the game
	 * @param message The message to apply
	 * @throws IOException 
	 */
	public void applyAction(GameActionMessage message) throws IOException
	{
		int senderId = message.getSenderId();
		if (senderId < 0 || senderId >= board.getPlayers().size())
		{
			return;
		}
		
		Player player = board.getPlayers().get(senderId);
		switch (message.getType())
		{
		case MOVING_FORWARD:
			player.setMotion(Player.MOTION_FORWARD);
			break;
		case MOVING_BACKWARD:
			player.setMotion(Player.MOTION_BACKWARD);
			break;
		case STOP_MOVING:
			player.setMotion(Player.MOTION_NONE);
			break;
		case TURNING_LEFT:
			player.setTurning(Player.TURNING_LEFT);
			break;
		case TURNING_RIGHT:
			player.setTurning(Player.TURNING_RIGHT);
			break;
		case STOP_TURNING:
			player.setTurning(Player.TURNING_NONE);
			break;
		case ACTIVATE_GEMS:
			//remove gems from players
			int data = message.getData();
			int[] gemArr = GameController.splitBitStream(data);
			boolean valid = player.verifyColors(gemArr);
			
			if (valid)
			{
				player.removeGems(gemArr);
				
				GameInfoMessage gameInfo = new GameInfoMessage(ServerBase.RESERVED_SERVER_ID, GameInfoMessage.GameInfoType.ATTACK_FIRED);
				gameInfo.setData1(senderId);
				gameInfo.setData2(data);
				gameInfo.setFacing(player.getFacing());
				gameInfo.setX(player.getX());
				gameInfo.setY(player.getY());
				server.serverMessage(gameInfo);
				
				// create a projectile
				int[] attack = GameController.parseAttackData(gemArr);
				if (attack[1] == 3)
				{
					// the player leveled up in a color
					player.setStatus(Player.CAST_STATUS, 1000);
					player.getLevels()[attack[0]]++;
				}
				else if (attack[1] == 2)
				{
					int level = player.getLevels()[attack[0]];
					if (attack[0] == 2)
					{
						player.setStatus(Player.HASTE_STATUS, 
								(int)(Projectile.EFFECT_DURATION[attack[0]] * (1 + level * 0.2f)));
					}
					else if (attack[0] == 4)
					{
						player.setStatus(Player.INVINCIBLE_STATUS,
								(int)(Projectile.EFFECT_DURATION[attack[0]] * (1 + level * 0.2f)));
					}
					else if (attack[0] == 5)
					{
						// someone got struck by lightning
						double mindistsq = Double.MAX_VALUE;
						int minindex = -1;
						for (int i = 0; i < board.getPlayers().size(); i++)
						{
							Player p = board.getPlayers().get(i);
							if (i == player.getIndex()) 
							{
								continue;
							}
							double dist = (p.getX() - player.getX()) * (p.getX() - player.getX())
								+ (p.getY() - player.getY()) * (p.getY() - player.getY());
							
							if (minindex == -1 || dist < mindistsq)
							{
								minindex = i;
								mindistsq = dist;
							}
						}
						
						board.getPlayers().get(minindex).setStatus(Player.LIGHTNING_STATUS, 
								(int)(Projectile.EFFECT_DURATION[attack[0]] * (1 + level * 0.2f)));
					}
					else
					{
						// the player fired a major attack
						Projectile proj = Projectile.createProjectile(attack[0], player);
						board.getProjectiles().add(proj);
					}
				}
			}
		}
	}
	
	public void update(long elapsedTime)
	{
		try
		{
			//change if there's no need for the game over flag
			if(gameOver){
				return;
			}
			
			//send the game over message to all clients
			if(remainingTime<=0)
			{	
				//set the players score in
				int scores[][] = new int[board.getPlayers().size()][Gem.NUM_GEMS];
				
				for(int i=0; i<board.getPlayers().size();i++){
					Player p = board.getPlayers().get(i);
					for(int j=0; j<p.getLevels().length;j++){
						scores[i][j] = p.getLevels()[j];
					}
				}
				
				GameOverMessage gom = new GameOverMessage(ServerBase.RESERVED_SERVER_ID);
				gom.setScores(scores);
				server.serverMessage(gom);
				gameOver = true;
			}
			
			generateGems(elapsedTime);
			remainingTime -= elapsedTime;
			timeToSendUpdates -= elapsedTime;
			
			for (int i = 0; i < board.getProjectiles().size(); i++)
			{
				Projectile proj = board.getProjectiles().get(i);
				proj.update(elapsedTime);
				if ((proj.getDuration() == Projectile.INFINITE_DURATION &&
						board.willCollideWithTerrain(proj.getX(), proj.getY(), proj.getRadius()))
						|| proj.getDuration() <= 0)
				{
					board.getProjectiles().remove(i);
					i--;
				}
			}
			
			for (int i = 0; i < board.getPlayers().size(); i++)
			{
				Player p = board.getPlayers().get(i);
				p.update(elapsedTime);
				
				// check if player has obtained a gem
				int gemIndex = board.collisionWithGem(p);
				if (gemIndex != -1 && p.nextEmptySlot() != -1)
				{
					Gem gem = board.getGems().remove(gemIndex);
					p.gainedGem(gem.getColor());
					GameInfoMessage gameInfo = new GameInfoMessage(ServerBase.RESERVED_SERVER_ID, 
							GameInfoMessage.GameInfoType.GEM_PICKED_UP);
					gameInfo.setData1(p.getIndex());
					gameInfo.setData2(gemIndex);
					gameInfo.setData3(gem.getColor());
					gameInfo.setX(gem.getX());
					gameInfo.setY(gem.getY());
					server.serverMessage(gameInfo);
				}
				
				int projIndex = board.collisionWithProjectiles(p);
				// check if a projectile hit the player
				if (projIndex != -1)
				{
					Projectile proj = board.getProjectiles().get(projIndex);
					if (proj.getIndex() == 6 && p.getStatus() != Player.INVINCIBLE_STATUS)
					{
						// player got hit by a flying bicep
						p.applyAcceleration(proj.getDx() * (3 + proj.getLevel()), 
								proj.getDy() * (3 + proj.getLevel()));
					}
					else if (p.getStatus() != proj.getIndex() && 
							p.getStatus() != Player.INVINCIBLE_STATUS)
					{
						p.setStatus(proj.getIndex(), proj.getEffectDuration());
					}
					
					if (proj.getDuration() == Projectile.INFINITE_DURATION)
					{
						proj = board.getProjectiles().remove(projIndex);
					}
				}
			}
			if (timeToSendUpdates <= 0)
			{
				timeToSendUpdates += UPDATE_INTERVAL;
				sendPlayerStateUpdates();	
			}
		}
		catch(Exception IOException)
		{
			LOGGER.info("Caught IOException under update in GameEngine.");
		}
	}
	
	/**
	 * This method will generate more gems if there are not enough available
	 * @throws IOException 
	 */
	private void generateGems(long elapsedTime) throws IOException
	{
		gemCreationTime -= elapsedTime;
		if (gemCreationTime <= 0)
		{
			gemCreationTime += GEM_CREATION_DELAY;
			double prob = Math.exp((NUM_START_GEMS - board.getGems().size()) / 5);
			if (rng.nextDouble() < prob)
			{
				boolean valid = false;
				while (!valid)
				{
					int nx = rng.nextInt(board.getWidth() - Gem.GEM_RADIUS * 2) + Gem.GEM_RADIUS;
					int ny = rng.nextInt(board.getHeight() - Gem.GEM_RADIUS * 2) + Gem.GEM_RADIUS;
					if (board.willCollide(nx, ny, Gem.GEM_RADIUS * 2))
					{
						continue;
					}
					if (board.willCollideWithPlayers(nx, ny, Player.PLAYER_RADIUS * 2, -1))
					{
						continue;
					}
					
					int ncolor = rng.nextInt(Gem.NUM_GEMS);
					Gem gem = new Gem(nx, ny, ncolor);
					board.getGems().add(gem);
					System.out.println("New gem generated at " + nx + "," + ny);
					GameInfoMessage gim = new GameInfoMessage(ServerBase.RESERVED_SERVER_ID, GameInfoType.GEM_CREATED);
					gim.setX(nx);
					gim.setY(ny);
					gim.setData1(ncolor);
					server.serverMessage(gim);
					valid = true;
				}
			}
		}
	}
	
	/**
	 * Sends updates to all players with the position and state of each player
	 */
	private void sendPlayerStateUpdates() 
	{
		PlayerStateMessage psm = new PlayerStateMessage(ServerBase.RESERVED_SERVER_ID);
		psm.setRemainingTime(remainingTime);
		for (int i = 0; i < board.getPlayers().size(); i++)
		{
			Player p = board.getPlayers().get(i);
			psm.getPlayerData().add(PlayerStateData.createPlayerStateData(p));
		}
		try 
		{
			server.serverMessage(psm);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
}
