package coloreti.data;

import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.io.*;
import java.util.*;

public class Board {
	
	private ArrayList<Gem> gems;
	private ArrayList<Player> players;
	private ArrayList<Polygon> boundaries;
	private ArrayList<Projectile> projectiles;
	private int width, height;
	
	public Board(int width, int height)
	{
		gems = new ArrayList<Gem>();
		players = new ArrayList<Player>();
		boundaries = new ArrayList<Polygon>();
		projectiles = new ArrayList<Projectile>();
		this.width = width;
		this.height = height;
	}
	
	public int getWidth()
	{
		return width;
	}
	
	public int getHeight()
	{
		return height;
	}
	
	/**
	 * Gets the list of gems
	 * @return The list of gems
	 */
	public ArrayList<Gem> getGems()
	{
		return gems;
	}
	
	/**
	 * Gets the list of players
	 * @return The list of players
	 */
	public ArrayList<Player> getPlayers()
	{
		return players;
	}
	
	/**
	 * Gets the list of projectiles
	 * @return The list of projectiles
	 */
	public ArrayList<Projectile> getProjectiles()
	{
		return projectiles;
	}
	
	/**
	 * Gets a list of collision boundaries
	 * @return The list of collision boundaries
	 */
	public ArrayList<Polygon> getBoundaries()
	{
		return boundaries;
	}
	
	public void loadBoundaries(String filename)
	{
		try
		{
			Scanner sc = new Scanner(new File(filename));
			Polygon poly = new Polygon();
			while (sc.hasNextLine())
			{
				String line = sc.nextLine();
				if (line.equals("Break"))
				{
					boundaries.add(poly);
					poly = new Polygon();
				}
				else
				{
					String[] pos = line.split(",");
					int x = Integer.parseInt(pos[0]);
					int y = Integer.parseInt(pos[1]);
					poly.addPoint(x, y);
				}
			}
			sc.close();
		}
		catch (IOException e)
		{
			e.printStackTrace();
			System.exit(0);
		}
	}
	
	/**
	 * Called by a player object to check if it will collide with anything on the
	 * board. Returns true if there will be a collision
	 * @param x The x position of the player
	 * @param y The y position of the player
	 * @param radius The radius of the player
	 * @return true if there will be a collision, false otherwise
	 */
	public boolean willCollide(float x, float y, float radius)
	{
		// for now, we are only checking if the player will go off the board
		if (x - radius < 1 || x + radius > width - 1 || y - radius < 1 || y + radius > height - 1)
		{
			return true;
		}
		if (willCollideWithTerrain(x, y, radius)) 
		{
			return true;
		}
		
		return false;
	}
	
	/**
	 * Called to check if a circular object will collide with any terrain objects
	 * @param x The x coordinate of the object
	 * @param y The y coordinate of the object
	 * @param radius The radius of the object
	 * @return true if there will be a collision, false otherwise
	 */
	public boolean willCollideWithTerrain(float x, float y, float radius)
	{
		double radiussq = radius * radius;
		for (int i = 0; i < boundaries.size(); i++)
		{
			Polygon poly = boundaries.get(i);
			if (poly.contains(x, y)) 
			{
				return true;
			}
			for (int j = 0; j < poly.npoints; j++)
			{
				int next = j == poly.npoints - 1 ? 0 : j + 1;
				if (Line2D.ptSegDistSq(poly.xpoints[j], poly.ypoints[j], 
						poly.xpoints[next], poly.ypoints[next], x, y) < radiussq)
				{
					return true;
				}
			}
		}
		
		return false;
	}
	
	public boolean willCollideWithPlayers(float x, float y, float radius, int index)
	{
		double radiussq = radius * radius;
		for (int i = 0; i < players.size(); i++)
		{
			if (i == index) continue;
			double dx = players.get(i).getX() - x;
			double dy = players.get(i).getY() - y;
			double distsq = dx * dx + dy * dy;
			if (distsq < radiussq) return true;
		}
		return false;
	}
	
	/**
	 * Checks if the player has collided with a gem, and if so, returns the index of the gem
	 * that the player collided with. This method assumes that the player can collide with
	 * at most one gem at any given time, and will return the first gem that has collided.
	 * @param p The player to check
	 * @return the index of the gem that has collided, or -1 if the player has not collided
	 * with any gem
	 */
	public int collisionWithGem(Player p)
	{
		for (int i = 0; i < gems.size(); i++)
		{
			float dx = gems.get(i).getX() - p.getX();
			float dy = gems.get(i).getY() - p.getY();
			// compute distance squared to avoid square root
			if (dx * dx + dy * dy < (Player.PLAYER_RADIUS + Gem.GEM_RADIUS) * (Player.PLAYER_RADIUS + Gem.GEM_RADIUS))
			{
				return i;
			}
		}
		return -1;
	}
	
	/**
	 * Checks for collision with projectiles
	 * @param p The player to check
	 * @return the index of the projectile that the player has collided with, or -1 if the
	 * player has not collided with any projectile
	 */
	public int collisionWithProjectiles(Player p)
	{
		for (int i = 0; i < projectiles.size(); i++)
		{
			if (projectiles.get(i).getOriginator() == p.getIndex())
			{
				continue;
			}
			float dx = projectiles.get(i).getX() - p.getX();
			float dy = projectiles.get(i).getY() - p.getY();
			float radius = Player.PLAYER_RADIUS + projectiles.get(i).getRadius();
				
			if (dx * dx + dy * dy < radius * radius)
			{
				return i;
			}
		}
		return -1;
	}
}
