package coloreti.data;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Arrays;

import coloreti.gui.GameScreen;
import coloreti.gui.GuiBase;
import coloreti.gui.Overlay;
import coloreti.interfaces.Displayable;

/**
 * This class represents a player object
 * Note that this displayable's point of reference is it's center.
 * 
 * @author ctheng
 * @author bsee
 *
 */
public class Player implements Displayable {

	private Board board;
	private float px, py;
	private float dx, dy;
	private float facing;
	private float velocity;
	private int[] levels;
	private int[] slots;

	private int index;
	private String name;
	
	private int motion;
	private int turning;
	private boolean activePlayer;
	
	private BufferedImage headImage,bodyImage;
	//head's index == bodyIndexY
	private int bodyIndexX, bodyIndexY;
	private float prevX, prevY;
	private float accumDistance;

	public final static int NUM_SLOTS = 5;
	public final static float CATCHUP_TIME = 100;
	public final static float CATCHUP_FACTOR = 0.00003f;
	public final static float BAILOUT_DISTANCE_SQ = 500 * 500;
	/** The ratio between backward motion and forward motion */
	public final static float BACKWARD_MOTION_PENALTY = 0.5f;
	/** The ratio between active deceleration and acceleration */
	public final static float DECEL_MULTIPLIER = 2;
	/** The rate of deceleration due to friction */
	public final static double FRICTION_RATE = 0.002f;
	/** The maximum distance a player can move in 1 ms */
	public final static float MAX_VELOCITY = 0.4f;
	/** Player can reach the maximum velocity of 200 px/sec in 600 ms */
	public final static float MAX_ACCEL = MAX_VELOCITY / 1000.0f;
	/** Player may turn 360 degrees in 2 secs */
	public final static double TURNING_RATE = Math.PI * 2 / 2000.0f;
	public final static int PLAYER_RADIUS = 22;
	public final static int PLAYER_COLLISION_RADIUS = 15;
	public final static int MOTION_FORWARD = 1;
	public final static int MOTION_NONE = 0;
	public final static int MOTION_BACKWARD = -1;
	public final static int TURNING_LEFT = -1;
	public final static int TURNING_RIGHT = 1;
	public final static int TURNING_NONE = 0;
	/** Distance travelled before the player takes the next step */
	public final static float DIST_PER_STEP = 50;

	/**status constants*/
	public final static int NO_STATUS = 8;
	public final static int CAST_STATUS = 7;
	public final static int FIRE_STATUS = 0;
	public final static int ICE_STATUS = 1;
	public final static int INVINCIBLE_STATUS = 2;
	public final static int CONFUSE_STATUS = 3;
	public final static int HASTE_STATUS = 4;
	public final static int LIGHTNING_STATUS = 5;
	
	public final static int[] OVERLAY_INDEX = {
		Overlay.FIRE_OVERLAY, Overlay.FREEZE_OVERLAY, Overlay.NO_OVERLAY, 
		Overlay.CONFUSE_OVERLAY, Overlay.HASTE_OVERLAY, Overlay.LIGHTNING_OVERLAY, 
		Overlay.NO_OVERLAY, Overlay.CAST_OVERLAY, Overlay.NO_OVERLAY
	};
	
	
	//overlay stuff
	private Overlay overlay;
	private long accumTime;	
	
	private boolean isCasting;
	
	//status stuff
	private int status;
	
	//offset of the player's head away from the body
	public final static int offsetY[][]=
	{	
		{0,0,-1,-1,0,-1,-2,-3},
		{0,2,-2,-1,-2,0,-3,-1},
		{2,5,-2,-1,-3,0,-3,1},
		{0,3,-1,-1,0,0,-3,1},
		
		{0,1,-1,-2,0,-2,-2,1},
		{-1,0,-1,-2,0,-1,-3,1},
		{-4,0,-1,-2,-1,-2,-3,1},
		{-4,-1,-3,-2,-1,-1,-4,-1}
	};
	
	//offset of the player's head away from the body
	public final static int offsetX[][]=
	{	
		{0,-1,0,1,1,-2,-1,0},
		{0,2,0,2,2,0,1,0},
		{1,4,2,2,2,0,-3,0},
		{1,-1,3,3,0,0,-2,0},
		
		{1,1,0,1,1,2,-1,0},
		{2,1,2,2,0,0,-2,-2},
		{2,-1,2,3,1,-1,-3,0},
		{0,1,3,2,1,0,-3,0}
	};
	
	public final static String headFileNames[]=
	{
		"textures/headPlain.gif",
		"textures/headPlain.gif",
		"textures/headPlain.gif",
		
		"textures/headPlain.gif",
		"textures/headPlain.gif",
		"textures/headPlain.gif"
	};
	
	public final static String bodyFileNames[]=
	{
		"sprites/bodyPlain.gif",
		"sprites/bodypink.gif",
		"sprites/bodyblue.gif",
		
		"sprites/bodygreen.gif",
		"sprites/bodyyellow.gif",
		"sprites/bodyred.gif"
	};
	
	public Player (float x, float y, Board board)
	{
		this.board = board;
		this.px = x;
		this.py = y;
		dx = dy = 0;
		facing = 0;
		velocity = 0;
		levels = new int[Gem.NUM_GEMS];
		slots = new int[NUM_SLOTS];
		Arrays.fill(slots, -1);
		activePlayer = false;
		
		bodyIndexX = 0;
		bodyIndexY = 0;
		prevX = px;
		prevY = py;
		accumDistance = 0;
		status = NO_STATUS;
		isCasting = false;
		
		overlay = new Overlay(this);
	}
	
	public Player(float x, float y, Board board, int index)
	{
		this(x, y, board);
		this.index = index;
		loadImages();
	}

	public void loadImages()
	{
		headImage = GuiBase.loadBufferedImage(headFileNames[index]);
		bodyImage = GuiBase.loadBufferedImage(bodyFileNames[index]);
	}

	/**
	 * @return the x
	 */
	public float getX() {
		return px;
	}

	/**
	 * @return the y
	 */
	public float getY() {
		return py;
	}

	/**
	 * @return the dx
	 */
	public float getDx() {
		return dx;
	}

	/**
	 * @return the dy
	 */
	public float getDy() {
		return dy;
	}
	
	/**
	 * Applies a change in velocity to the player
	 * @param ddx The change in dx
	 * @param ddy The change in dy
	 */
	public void applyAcceleration(float ddx, float ddy)
	{
		dx += ddx;
		dy += ddy;
	}
	
	/**
	 * @return the facing
	 */
	public float getFacing() {
		return facing;
	}

	/**
	 * @return the velocity
	 */
	public float getVelocity() {
		return velocity;
	}

	/**
	 * Gets the levels
	 * @return The levels in the different gem colors as an array
	 */
	public int[] getLevels()
	{
		return levels;
	}
	
	/**
	 * Gets the type of gem in each slot
	 * @return The slots array
	 */
	public int[] getSlots()
	{
		return slots;
	}
	
	/**
	 * Gets the player's index
	 * @return the index of the player
	 */
	public int getIndex()
	{
		return index;
	}

	/**
	 * Sets the player's index
	 * @param index The index of the player
	 */
	public void setIndex(int index)
	{
		this.index = index;
	}
	
	/**
	 * Gets the player's name
	 * @return The name
	 */
	public String getName()
	{
		return name;
	}
	
	/**
	 * Sets the player's name
	 * @param name The name
	 */
	public void setName(String name)
	{
		this.name = name;
	}
	
	/**
	 * @return the motion
	 */
	public int getMotion() {
		return motion;
	}

	/**
	 * @return the turning
	 */
	public int getTurning() {
		return turning;
	}

	/**
	 * @param motion the motion to set
	 */
	public void setMotion(int motion) {
		this.motion = motion;
	}

	/**
	 * @param turning the turning to set
	 */
	public void setTurning(int turning) {
		this.turning = turning;
	}
	
	/**
	 * Checks if this player is the active player
	 * @return true if this player is active
	 */
	public boolean isActivePlayer()
	{
		return activePlayer;
	}
	
	/**
	 * Sets if this player is the active player
	 * @param active true if this player is active
	 */
	public void setActivePlayer(boolean active)
	{
		activePlayer = active;
	}
	
	/**
	 * Returns the index of the next empty slot, or -1 if all slots are full
	 * @return The next empty slot
	 */
	public int nextEmptySlot()
	{
		for (int i = 0; i < slots.length; i++)
		{
			if (slots[i] == -1) return i;
		}
		return -1;
	}
	
	/**
	 * This method is called when the player has gained a gem
	 * @param color The color of gem gained
	 */
	public void gainedGem(int color)
	{
		int slot = nextEmptySlot();
		if (slot == -1) return;
		slots[slot] = color;
	}

	/**
	 * This method gets data needed to initialize a player
	 * @return The player init data
	 */
	public PlayerInitData getPlayerInitData()
	{
		PlayerInitData pid = new PlayerInitData(px, py, index);
		return pid;
	}
	
	/**
	 * This applies an update from the server to the client's representation of 
	 * the player. Basically, it modifies the velocity of the player so that
	 * he can catch up with himself on the server. However, if the player is too
	 * far away (beyond a certain "bail out" distance), the player will just be
	 * teleported to the correct position
	 * @param psd The player state data
	 */
	public void applyUpdate(PlayerStateData psd)
	{
		facing = psd.getFacing();
		motion = psd.getMotion();
		turning = psd.getTurning();
		
		float x = psd.getX();
		float y = psd.getY();
		float distsq = (x - px) * (x - px) + (y - py) * (y - py);
		
		if (distsq > BAILOUT_DISTANCE_SQ)
		{
			px = psd.getX();
			py = psd.getY();
			dx = psd.getDx();
			dy = psd.getDy();
		}
		else
		{
			x += CATCHUP_TIME * psd.getDx();
			y += CATCHUP_TIME * psd.getDy();
			dx += (x - px) * CATCHUP_FACTOR;
			dy += (y - py) * CATCHUP_FACTOR;
		}
		
	}
	
	@Override
	public void drawComponent(Graphics2D g, int x, int y)
	{
		int facingDir = (int)Math.round(4 * facing / Math.PI);
		while (facingDir < 0) facingDir += 8;
		while (facingDir > 8) facingDir -= 8;

		//draws the sprite
		bodyIndexY = (facingDir+6)%8;
		if (status != INVINCIBLE_STATUS || accumTime % 800 < 400)
		{
			drawBody(g, Math.round(px + x) - 25, Math.round(py + y) - 60);
		}
		if (status != NO_STATUS)
		{
			overlay.drawComponent(g, x, y);
		}
		g.setFont(new Font("Verdana", Font.BOLD, 14));
		// draw the player's name last
		GameScreen.drawCenteredString(g, Math.round(px + x), Math.round(py + y) + 45, name);
	}

	@Override
	public void update(long elapsedTime) {
		updatePlayerMotion(elapsedTime);
		
		accumTime -= elapsedTime;
		
		//to handle both casting and attacked animations
		if (status != NO_STATUS && accumTime <= 0)
		{
			accumTime = 0;
			status = NO_STATUS;
			overlay.castOverlay(OVERLAY_INDEX[status]);

		}
		if(status != NO_STATUS){
			overlay.update(elapsedTime);
		}
		
		overlay.update(elapsedTime);

	}
	
	/**
	 * Draws the body and head sprites
	 * @param g The graphics context
	 * @param x The x coordinate of the top left reference point
	 * @param y The y coordinate of the top left reference point
	 */
	public void drawBody(Graphics2D g, int x, int y)
	{
		int subX, subY;
		subX = bodyIndexX * 50;
		subY = bodyIndexY * 100;
		Image currBody = bodyImage.getSubimage(subX, subY, 50, 100);
		Image headBody = headImage.getSubimage(0, subY, 50, 100);
		g.drawImage(currBody, x, y, null);
		g.drawImage(headBody, x+offsetX[bodyIndexX][bodyIndexY], y+offsetY[bodyIndexX][bodyIndexY], null);
	}
	
	/**
	 * Updates the player's motion
	 * @param elapsedTime the elapsed time since the last call
	 */
	public void updatePlayerMotion(long elapsedTime)
	{
		float displaceX = px - prevX;
		float displaceY = py - prevY;
		int effectiveMotion = status == CONFUSE_STATUS ? -motion : motion;
		int effectiveTurning = status == CONFUSE_STATUS ? -turning : turning;
		
		// deal with 
		double euclideanDist = Math.sqrt(displaceX*displaceX + displaceY*displaceY);
		accumDistance += euclideanDist;
		if(accumDistance >= DIST_PER_STEP)
		{
			bodyIndexX += motion;
			if (bodyIndexX < 0) bodyIndexX += 8;
			if (bodyIndexX >= 8) bodyIndexX -= 8;
			accumDistance -= DIST_PER_STEP;
		}
		prevX = px;
		prevY = py;
		
		// calculate component of velocity
		double ux = Math.cos(facing);
		double uy = Math.sin(facing);
		double dot = dx * ux + dy * uy;
		
		if (status == FIRE_STATUS || status == LIGHTNING_STATUS) 
		{
			dx = 0;
			dy = 0;
			return;
		}
		
		if (effectiveMotion == 1)
		{
			if (dot >= 0)
			{
				// motion and velocity match, so speed up
				dx += ux * MAX_ACCEL * elapsedTime;
				dy += uy * MAX_ACCEL * elapsedTime;
			}
			else
			{
				// apply deceleration
				dx += ux * MAX_ACCEL * DECEL_MULTIPLIER * elapsedTime;
				dy += uy * MAX_ACCEL * DECEL_MULTIPLIER * elapsedTime;
			}
		}
		else if (effectiveMotion == -1)
		{
			if (dot >= 0)
			{
				// apply deceleration
				dx += -ux * MAX_ACCEL * DECEL_MULTIPLIER * elapsedTime;
				dy += -uy * MAX_ACCEL * DECEL_MULTIPLIER * elapsedTime;
			}
			else
			{
				// apply acceleration in the backward motion
				dx += -ux * MAX_ACCEL * BACKWARD_MOTION_PENALTY * elapsedTime;
				dy += -uy * MAX_ACCEL * BACKWARD_MOTION_PENALTY * elapsedTime;
			}
		}
		
		// now deal with change in turning
		if (effectiveTurning != 0)
		{
			facing += effectiveTurning * elapsedTime * TURNING_RATE;
			if (facing > 2 * Math.PI)
			{
				facing -= 2 * Math.PI;
			}
			if (facing < -2 * Math.PI)
			{
				facing += 2 * Math.PI;
			}
		}
		
		// now slow down the user proportionately to his current speed
		double speedModifier = 1;
		if (status == ICE_STATUS)
		{
			speedModifier = 2;
		}
		else if (status == HASTE_STATUS)
		{
			speedModifier = 0.5;
		}
	
		dx -= speedModifier * FRICTION_RATE * elapsedTime * dx;
		dy -= speedModifier * FRICTION_RATE * elapsedTime * dy;
	
		float nx = px + dx * elapsedTime;
		float ny = py + dy * elapsedTime;
		
		// staggered collision checking for smooth movement
		if (!board.willCollide(nx, ny, PLAYER_COLLISION_RADIUS) && 
				!board.willCollideWithPlayers(nx, ny, PLAYER_RADIUS, index))
		{
			px = nx;
			py = ny;
		}
		else
		{
			if (!board.willCollide(nx, py, PLAYER_COLLISION_RADIUS)
				&& !board.willCollideWithPlayers(nx, py, PLAYER_RADIUS, index))
			{
				px = nx;
			}
			else
			{
				if (!board.willCollide(px, ny, PLAYER_COLLISION_RADIUS)
					&& !board.willCollideWithPlayers(px, ny, PLAYER_RADIUS, index))
				{
					py = ny;
				}
			}
		}
	}

	/**
	 * This method removes the gems in the gems array from the player's slots. 
	 * It is assumed that the player possesses the gems that are removed. There is
	 * no effect if the player does not have one of the gems specified in the gems
	 * array
	 * @param gems The array of gems to remove
	 */
	public void removeGems(int[] gems)
	{
		int count = 0;
		for (int i = 0; i < gems.length; i++)
		{
			for (int j = 0; j < slots.length; j++)
			{
				if (gems[i] == slots[j])
				{
					slots[j] = -1;
					count++;
					break;
				}
			}
		}
	}
	
	public void startCasting()
	{
		accumTime = 0;
		isCasting = true;
		overlay.castOverlay(OVERLAY_INDEX[CAST_STATUS]);
	}
	
	//returns the number of valid gems, only in the activate slot
	public int numGems()
	{
		int count = 0;
		for(int i=2; i<slots.length; i++)
		{
			if(slots[i] != -1){
				count++;
			}
		}
		return count;
		
	}
	
	//returns the data of the active gems
	public int bitStreamActiveGems()
	{
		int data=0;
		for(int i=2; i<slots.length; i++)
		{
			data = data<<3;
			data += slots[i];
			//since we only have 7 gems, we shift by 3
		}
		
		return data;
	}

	public int getStatus() 
	{
		return status;
	}
	
	/**
	 * Returns the duration of time the player is afflicted by the current status,
	 * or -1 if there is no current status
	 * @return The remaining status duration
	 */
	public int getStatusDuration()
	{
		if (status == NO_STATUS)
		{
			return -1;
		}
		return (int) accumTime;
	}

	//slightly modified
	public void setStatus(int status, int duration) 
	{
		accumTime = duration;
		overlay.castOverlay(OVERLAY_INDEX[status]);
		this.status = status;
	}
	
	/**
	 * Verifies if the player has gems of the colors in the gem array
	 * @param gems The gems to check
	 * @return true if the player has gems of these colors
	 */
	public boolean verifyColors(int[] gems)
	{
		ArrayList<Integer> list = new ArrayList<Integer>();
		for (Integer i : slots)
		{
			list.add(i);
		}
		
		for (Integer gem : gems)
		{
			if (list.contains(gem))
			{
				list.remove(gem);
			}
			else 
			{
				return false;
			}
		}
		return true;
	}
	
}
