package coloreti.data;

import java.awt.*;

import coloreti.GameController;
import coloreti.gui.GuiBase;
import coloreti.interfaces.Displayable;

public class Projectile implements Displayable 
{
	public static final String[] projectileFilenames = new String[] {
		"textures/fireball.png",
		"textures/ringofice.png",
		"none",
		"textures/purpleorb.png",
		"none",
		"none",
		"textures/cannonball.png"
	};
	
	public static final int INFINITE_DURATION = 100000000;
	public static Image[] projectileImages = new Image[projectileFilenames.length];
	public static final int[] BASE_RADIUS = new int[] {
		25, 100, 25, 30, 25, 25, 20
	};
	
	public static final int[] INCREMENT_RADIUS = new int[] {
		8, 33, 5, 10, 5, 5, 8
	};
	
	public static final float[] INITIAL_VELOCITY = new float[] {
		0.3f, 0, 0.3f, 0.4f, 0.3f, 0.3f, 0.4f
	};
	
	public static final int[] PROJECTILE_DURATION = new int[] {
		INFINITE_DURATION, 2000, INFINITE_DURATION, INFINITE_DURATION, INFINITE_DURATION,
		INFINITE_DURATION, INFINITE_DURATION
	};
	      
	public static final int[] EFFECT_DURATION = new int[] {
		3500, 4000, 6000, 6000, 6000, 2500, 2000
	};
	    
	static
	{
		for (int i = 0; i < projectileImages.length; i++)
		{
			projectileImages[i] = GuiBase.loadImage(projectileFilenames[i]);
		}
	}
	
	
	private float px, py;
	private float dx, dy;
	private int radius;
	private int index;
	private int duration;
	private int level;
	private int originator;
	private Image projectileImage;
	
	public Projectile(float x, float y, int radius, int index)
	{
		this.px = x;
		this.py = y;
		this.dx = dx;
		this.dy = dy;
		this.radius = radius;
		this.index = index;
		projectileImage = projectileImages[index].getScaledInstance(radius * 2, radius * 2, Image.SCALE_SMOOTH);
	}
	
	public Projectile (float x, float y, float dx, float dy, int radius, 
			int index, int level, int duration, int originator)
	{
		this.px = x;
		this.py = y;
		this.dx = dx;
		this.dy = dy;
		this.radius = radius;
		this.index = index;
		this.level = level;
		projectileImage = projectileImages[index].getScaledInstance(radius * 2, radius * 2, Image.SCALE_SMOOTH);
		this.duration = duration;
		this.originator = originator;
	}
	
	/**
	 * @return the x
	 */
	public float getX() {
		return px;
	}

	/**
	 * @return the y
	 */
	public float getY() {
		return py;
	}

	/**
	 * @return the dx
	 */
	public float getDx() {
		return dx;
	}
	
	public void setDx(float dx)
	{
		this.dx = dx;
	}

	/**
	 * @return the dy
	 */
	public float getDy() {
		return dy;
	}

	public void setDy(float dy)
	{
		this.dy = dy;
	}
	
	/**
	 * @return the radius
	 */
	public int getRadius() {
		return radius;
	}
	
	/**
	 * @return the index
	 */
	public int getIndex() {
		return index;
	}

	/**
	 * @return the level
	 */
	public int getLevel() {
		return level;
	}
	
	public void setLevel(int level)
	{
		this.level = level;
	}

	@Override
	public void drawComponent(Graphics2D g, int x, int y) 
	{
		g.drawImage(projectileImage, Math.round(px + x) - radius, Math.round(py + y) - radius, null);
	}

	@Override
	public void update(long elapsedTime) 
	{
		if (duration > 0 && duration < INFINITE_DURATION) duration -= elapsedTime;
		px += dx * elapsedTime;
		py += dy * elapsedTime;
	}

	/**
	 * @param duration the duration to set
	 */
	public void setDuration(int duration) {
		this.duration = duration;
	}

	/**
	 * @return the duration
	 */
	public int getDuration() {
		return duration;
	}
	
	/**
	 * Gets the duration of the effect if the projectile hits the player
	 * @return the duration of the effect
	 */
	public int getEffectDuration()
	{
		return (int) Math.round(EFFECT_DURATION[index] * (1 + level * 0.2f));
	}

	/**
	 * @param originator the originator to set
	 */
	public void setOriginator(int originator) {
		this.originator = originator;
	}

	/**
	 * @return the originator
	 */
	public int getOriginator() {
		return originator;
	}

	/**
	 * Creates a projectile given an attack color and the originating player
	 * @param color The type of the projectile
	 * @param player The originating player
	 * @return The projectile that was fired
	 */
	public static Projectile createProjectile (int color, Player player)
	{
		int level = player.getLevels()[color];
		int radius = BASE_RADIUS[color] + level * INCREMENT_RADIUS[color];
		Projectile proj = new Projectile(player.getX(), player.getY(), radius, color);
		proj.setOriginator(player.getIndex());
		proj.setLevel(level);
		proj.setDx(INITIAL_VELOCITY[color] * (float)Math.cos(player.getFacing()));
		proj.setDy(INITIAL_VELOCITY[color] * (float)Math.sin(player.getFacing()));
		proj.setDuration(PROJECTILE_DURATION[color]);
		return proj;
	}
	
}
