package coloreti.gui;

import java.awt.*;
import java.awt.image.*;

import java.util.*;

import coloreti.GameController;
import coloreti.data.Board;
import coloreti.data.Gem;
import coloreti.data.Player;
import coloreti.data.Projectile;
import coloreti.interfaces.Displayable;
import coloreti.util.Pair;

public class GameScreen implements Displayable {
	
	private static final String GEM_PANEL_FILENAME = "textures/gempanel2.jpg";
	private static final int[] GEM_PANEL_OFFSET = {15, 57, 113, 155, 197};
	private static final int GEM_PANEL_X = 15;
	private static final int GEM_PANEL_Y = 15;
	private static final Comparator<Pair<Float, Displayable>> acomp = 
		Pair.<Float, Displayable>getComparatorByA();
	
	private Board board;
	private BufferedImage bgImage;
	
	private Image gemPanel;
	
	private int currentPlayer;
	private int height, width;
	private int inactiveSelect;
	private int activeSelect;
	
	private GameController gameController;
	
	/**
	 * Creates a new game screen to display the board and UI
	 * @param board The board 
	 * @param width The width of the screen
	 * @param height The height of the screen
	 */
	public GameScreen(Board board, int width, int height, GameController gameController)
	{
		this.board = board;
		this.width = width;
		this.height = height;
		loadBgImage();
		gemPanel = GuiBase.loadImage(GEM_PANEL_FILENAME);
		inactiveSelect = activeSelect = -1;
		
		this.gameController = gameController;
	}
	
	/**
	 * Prepares and loads the background image
	 */
	private void loadBgImage()
	{
		bgImage = new BufferedImage(board.getWidth(), board.getHeight(), BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = bgImage.createGraphics();
		g.setBackground(Color.white);
		g.clearRect(0, 0, bgImage.getWidth(), bgImage.getHeight());
		Image map = GuiBase.loadImage("textures/map.jpg");
		g.drawImage(map, 0, 0, null);
		
		g.dispose();
	}
	
	/**
	 * Gets the current player's index
	 * @return The index of the current player
	 */
	public int getCurrentPlayer()
	{
		return currentPlayer;
	}
	
	/**
	 * Sets the current player's index
	 * @param player The index of the current player
	 */
	public void setCurrentPlayer(int player)
	{
		currentPlayer = player;
	}
	
	/**
	 * Selects a gem slot. Slots 1 and 2 are inactive slots, slots 3 - 5 are active slots.
	 * Only 1 inactive and 1 active slot can be selected at a time.
	 * @param slot The slot to select 
	 */
	public void selectGem(int slot)
	{
		if (slot < 1 || slot > 5) return;
		if (activeSelect == slot)
		{
			activeSelect = -1;
			return;
		}
		if (inactiveSelect == slot)
		{
			inactiveSelect = -1;
			return;
		}
		if (inactiveSelect == -1)
		{
			inactiveSelect = slot;
		}
		else
		{
			activeSelect = slot;
		}
		
	}
	
	/**
	 * Swaps the active and inactive gem, if both are selected
	 */
	public void swapGems()
	{
		if (activeSelect == -1 || inactiveSelect == -1)
		{
			return;
		}
		int[] slots = board.getPlayers().get(currentPlayer).getSlots();
		int temp = slots[activeSelect - 1];
		slots[activeSelect - 1] = slots[inactiveSelect - 1];
		slots[inactiveSelect - 1] = temp;
	}
	
	/**
	 * Gets the active slot selection
	 * @return the active slot selection
	 */
	public int getActiveSelect()
	{
		return activeSelect;
	}
	
	/**
	 * Gets the inactive slot selection
	 * @return the inactive slot selection
	 */
	public int getInactiveSelect()
	{
		return inactiveSelect;
	}
	
	/**
	 * Checks if both the inactive and active slots are selected
	 * @return true if there are 2 slots selected, false otherwise
	 */
	public boolean areTwoSlotsSelected()
	{
		return activeSelect != -1 && inactiveSelect != -1;
	}
	
	@Override
	public void drawComponent(Graphics2D g, int x, int y) {
		// calculate view point
		Player player = board.getPlayers().get(currentPlayer);
		float centerx = player.getX();
		float centery = player.getY();
		
		int leftx = Math.round(Math.max(0, centerx - width/2));
		int topy = Math.round(Math.max(0, centery - height/2));
		
		leftx = Math.min(leftx, board.getWidth() - width);
		topy = Math.min(topy, board.getHeight() - height);
		
		// draw background
		g.drawImage(bgImage.getSubimage(leftx, topy, width, height), x, y, null);
		
		// sort all objects by y coordinate
		ArrayList<Pair<Float, Displayable>> objectList = new ArrayList<Pair<Float, Displayable>>();
		for (Player p : board.getPlayers())
		{
			objectList.add(new Pair<Float, Displayable>(p.getY(), p));
		}
		
		for (Gem gem : board.getGems())
		{
			objectList.add(new Pair<Float, Displayable>(gem.getY(), gem));
		}
		Collections.sort(objectList, acomp);
		
		// draw them all
		for (Pair<Float, Displayable> pair : objectList)
		{
			pair.getB().drawComponent(g, -leftx + x, -topy + y);
		}
		
		for (int i = 0; i < board.getProjectiles().size(); i++)
		{
			Projectile proj = board.getProjectiles().get(i);
			proj.drawComponent(g, -leftx + x, -topy + y);
		}
		
		// draw direction dots for current player
		g.setColor(new Color(255, 236, 139));
		for (int i = 1; i < 4; i++)
		{
			int dist = i * 20 + Player.PLAYER_RADIUS;
			int xpos = (int)Math.round(player.getX() - leftx + x + dist * Math.cos(player.getFacing()));
			int ypos = (int)Math.round(player.getY() - topy + y + dist * Math.sin(player.getFacing()));
			g.fillOval(xpos - 3, ypos - 3, 6, 6);
		}
	
		// draw overlay
		g.drawImage(gemPanel, GEM_PANEL_X + x, GEM_PANEL_Y + y, null);
		for (int i = 0; i < Player.NUM_SLOTS; i++)
		{	
			int color = player.getSlots()[i];
			if (color == -1) continue;
			g.drawImage(Gem.getGemImage(color), GEM_PANEL_OFFSET[i] + GEM_PANEL_X, GEM_PANEL_Y + 16, null);
		}
		
		// draw selection, if any
		if (inactiveSelect != -1)
		{
			g.setColor(new Color(127, 127, 127));
			g.setStroke(new BasicStroke(2));
			g.drawRect(GEM_PANEL_X + GEM_PANEL_OFFSET[inactiveSelect - 1] - 2, GEM_PANEL_Y + 14, Gem.GEM_RADIUS * 2 + 4, Gem.GEM_RADIUS * 2 + 4);
		}
		
		if (activeSelect != -1)
		{
			g.setColor(new Color(64, 64, 64));
			g.setStroke(new BasicStroke(2));
			g.drawRect(GEM_PANEL_X + GEM_PANEL_OFFSET[activeSelect - 1] - 2, GEM_PANEL_Y + 14, Gem.GEM_RADIUS * 2 + 4, Gem.GEM_RADIUS * 2 + 4);
		}
		
		// draw score
		g.setFont(new Font("Verdana", Font.BOLD, 14));
		for (int i = 0; i < Gem.NUM_GEMS; i++)
		{
			g.setColor(Gem.gemColors[i]);
			g.drawString(String.valueOf(player.getLevels()[i]), 760, 20 + i * 18);
		}
		
		//draw time left
		long time = gameController.getRemainingTime();
		time = time / 1000;
		int minutes = (int)time / 60;
		int seconds = (int)time % 60;
		String timeString = minutes + ":" + seconds; 
		if (seconds < 10)
		{
			timeString = minutes + ":0" + seconds;
		}
		g.setFont(new Font("Verdana", Font.BOLD, 18));
		g.setColor(new Color(255, 255, 255));
		g.drawString(timeString, width - 50, height - 30);
	}

	@Override
	public void update(long elapsedTime) {
		// TODO Auto-generated method stub
		
	}
	
	public void mouseQuery(int x, int y)
	{
		Player player = board.getPlayers().get(currentPlayer);
		float centerx = player.getX();
		float centery = player.getY();
		
		int leftx = Math.round(Math.max(0, centerx - width/2));
		int topy = Math.round(Math.max(0, centery - height/2));
		
		leftx = Math.min(leftx, board.getWidth() - width);
		topy = Math.min(topy, board.getHeight() - height);
		
		System.out.println((leftx + x) + "," + (topy + y));
	}
	
	public static void drawCenteredString(Graphics2D g, int x, int y, String s)
	{
		int offset = g.getFontMetrics().stringWidth(s);
		g.drawString(s, x - offset/2, y);
	}
	
}
