package coloreti.gui;

import java.awt.*;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import java.awt.image.CropImageFilter;
import java.awt.image.FilteredImageSource;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JFrame;

import coloreti.interfaces.Displayable;
import coloreti.interfaces.MouseController;


/**
 * Formally MainFrame, this class is the new main class for any GUI application
 * 
 * GuiBase implements a windowed frame where components can be added. All components
 * will follow a nested structure, with each component maintaining a reference to its
 * parent component.
 * 
 * Components will be drawn to screen in the order in which they were added. A component
 * added later will be drawn above previous components, if multiple components overlap.
 * Each component is responsible for drawing its own children, and should follow the
 * above convention.
 * 
 * @author ctheng
 *
 */
@SuppressWarnings("serial")
public class GuiBase extends JFrame {

	private static int DEFAULT_HEIGHT = 600;
	private static int DEFAULT_WIDTH = 800;
	protected boolean isRunning;
	/** This stores the displayable objects */
	private ArrayList<Displayable> componentArray;
	/** This stores the location of the displayable objects */
	private ArrayList<Point> componentLocs;
	private BufferedImage bgImage;
	private BufferedImage bgImage2;
	private boolean selector;
	private long currentTime;
	
	public GuiBase(String title){
		this(title, DEFAULT_WIDTH, DEFAULT_HEIGHT);
	}
	
	public GuiBase(String title, int width, int height){
		super(title);
		this.setSize(width + 4, height + 31); // offsets to account for border size
		this.setVisible(true);
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		this.setResizable(false);
		this.setBackground(Color.white);
		bgImage = new BufferedImage(width + 2, height + 29, BufferedImage.TYPE_INT_ARGB);
		bgImage2 = new BufferedImage(width + 2, height + 29, BufferedImage.TYPE_INT_ARGB);
		isRunning = false;
		selector = false;
		componentArray = new ArrayList<Displayable>();
		componentLocs = new ArrayList<Point>();
		init();
	}
	
	/**
	 * Methods which will be called from the constructor. Override to
	 * add your own methods. 
	 */
	public void init(){
		
	}
	
	/**
	 * Starts the application
	 */
	public final void run(){
		isRunning = true;
		currentTime = System.currentTimeMillis();
		long accumTime = 0;
		Graphics2D g1 = bgImage.createGraphics();
		Graphics2D g2 = bgImage2.createGraphics();
		while (isRunning){
			
            // draw the screen
			Graphics g = this.getGraphics();
            if (selector){
            	draw(g1);
            	g.drawImage(bgImage, 2, 29, null);
            } else {
            	draw(g2);
            	g.drawImage(bgImage2, 2, 29, null);
            }
            selector = !selector;
            g.dispose();
			
			long elapsedTime = System.currentTimeMillis() - currentTime;
            currentTime += elapsedTime;
            accumTime += elapsedTime;
            long time = 0;
			while (accumTime > 20)
			{
				time += 20;
				accumTime -= 20;
			}
			if (time == 0) continue;
            // update
            update(time);

		}
	}
	
	/**
	 * Adds a mouse controller to the mainframe. A mouse controller is Displayable
	 * as well as implementing MouseListener and MouseMotionListener
	 * @param m The MouseController to add
	 */
	public void addMouseController(MouseController m){
		addDisplayable(m, 0, 0);
		addMouseListener(m);
		addMouseMotionListener(m);
	}
	
	public void addKeyController(KeyListener k){
		addKeyListener(k);
	}
	
	/**
	 * Adds a Displayable object to the mainframe. This object will be regularly 
	 * refreshed and drawn to screen. This constructor assumes the default location
	 * of 0,0.
	 * @param d The displayable object to add
	 */
	public void addDisplayable(Displayable d){
		addDisplayable(d, 0, 0);
	}
	
	/**
	 * Adds a Displayable object to the mainframe at a specified location. This object 
	 * will be regularly refreshed and drawn to screen. 
	 * @param d The displayable object to add
	 * @param x The x-coordinate of the leftmost pixel
	 * @param y The y-coordinate of the topmost pixel
	 */
	public void addDisplayable(Displayable d, int x, int y){
		componentArray.add(d);
		componentLocs.add(new Point(x,y));
	}
	
	
	/**
	 * Gets the location of a particular displayable object with respect to the main frame.
	 * @param componentNumber The component number of the object to get. This represents the
	 * 						  0-based numerical order that this Displayable was added.  
	 * @return A point representing the x,y coordinates of the top left of the displayed object
	 */
	public Point getLocationOf (int componentNumber){
		Point p = componentLocs.get(componentNumber);
		return new Point(p.x, p.y);
	}
	
	/**
	 * Changes the location of a particular displayable object with respect to the main frame.
	 * @param componentNumber The component number of the object to get. This represents the
	 * 						  0-based numerical order that this Displayable was added. 
	 * @param newX The new x-coordinate of the leftmost pixel
	 * @param newY The new y-coordinate of the rightmost pixel
	 */
	public void changeLocationOf (int componentNumber, int newX, int newY){
		Point p = componentLocs.get(componentNumber);
		p.setLocation(newX, newY);
	}
	
	/**
	 * Removes a displayable object from the screen
	 * @param componentNumber The component number of the object to get. This represents the
	 * 						  0-based numerical order that this Displayable was added. 
	 */
	public void removeDisplayable(int componentNumber){
		componentArray.remove(componentNumber);
		componentLocs.remove(componentNumber);
	}
	
	
	/**
	 * Updates all components in the component array.
	 * @param elapsedTime The time elapsed since last update
	 */
	protected void update(long elapsedTime){
		for (int i = 0; i < componentArray.size(); i++){
			componentArray.get(i).update(elapsedTime);
		}
	}
	
	/**
	 * Draws the components in the component array to the screen
	 * @param g The graphics context to draw on
	 */
	protected void draw(Graphics2D g){
		for (int i = 0; i < componentArray.size(); i++){
			componentArray.get(i).drawComponent(g, componentLocs.get(i).x, componentLocs.get(i).y);
		}
	}
	
	/**
	 * Pauses or unpauses the game.
	 *
	 */
	public void pause(){
		isRunning = !isRunning;
		currentTime = System.currentTimeMillis();
	}
	
	/**
	 * Loads an image from a file
	 * @param filename The filename to load from
	 * @return The loaded image
	 */
	public static Image loadImage(String filename)
	{
		return new ImageIcon(filename).getImage();
	}
	
	/**
	 * Loads an image from a file into a buffered image
	 * @param filename The filename to load from
	 * @return The loaded buffered image
	 */
	public static BufferedImage loadBufferedImage(String filename)
	{
		Image image = loadImage(filename);
		// Create a buffered image with a format that's compatible with the screen
        BufferedImage bimage = null;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try {
            // Determine the type of transparency of the new buffered image
            int transparency = Transparency.BITMASK;
    
            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bimage = gc.createCompatibleImage(
                image.getWidth(null), image.getHeight(null), transparency);
        } catch (HeadlessException e) {
            // The system does not have a screen
        }
    
        if (bimage == null) {
            // Create a buffered image using the default color model
            int type =  BufferedImage.TYPE_INT_ARGB;
            bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
        }
    
        // Copy image to buffered image
        Graphics g = bimage.createGraphics();
    
        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
    
        return bimage;

	}
	
}
