package coloreti.network.client;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Scanner;

import coloreti.GameController;
import coloreti.network.messages.*;
import coloreti.network.messages.InitialMessage.InitialMessageType;
import coloreti.network.exceptions.*;

public class ClientBase implements Runnable {
	
	private GameController parent;
	private int port;
	private String hostname;
	private Socket server;
	private ObjectInputStream in;
	private ObjectOutputStream out;
	private boolean running;
	private int id;
	private boolean connected;

	/**
	 * Creates a new client to the specified host name and port
	 * @param port
	 * @param hostname
	 */
	public ClientBase(GameController parent)
	{
		this.parent = parent;
		server = null;
		in = null;
		out = null;
		running = false;
		id = -1;
	}

	public void initialize()
	{
		
	}
	
	/**
	 * Sets this client's id
	 * @param id The id of the client
	 */
	public void setId(int id)
	{
		this.id = id;
	}
	
	/**
	 * Gets this client's id
	 * @return The id of the client
	 */
	public int getId()
	{
		return id;
	}
	
	/**
	 * Connects to the server
	 * @param hostname The host name
	 * @param port The port number
	 * @return true if successfully connected, false otherwise
	 */
	public boolean connect(String hostname, int port)
	{
		try
		{
			server = new Socket(hostname, port);
			out = new ObjectOutputStream(server.getOutputStream());
			out.flush();
			in = new ObjectInputStream(server.getInputStream());
		}
		catch (UnknownHostException e) 
		{
			System.err.println("Cannot find host " + hostname);
		} 
		catch (IOException e) 
		{
			System.err.println("I/O error connecting to the host " + hostname);
		}

		if (server == null || in == null || out == null) return false;
		System.out.println("Connected to server!");
		connected = true;
		return true;
	}
	
	public void run()
	{
		if (!connected)
		{
			return;
		}
		startClient();
		
		Message msg;
		try {
			while (true) 
			{
				msg = readMessage();
				if (msg == null) continue;
				parent.messageReceived(msg);
			}
				
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
			if (!server.isConnected())
			{
				stopClient();
			}
		} 
		catch (SevereException e) 
		{
			e.printStackTrace();
			stopClient();
		}
	}

	
	/**
	 * Returns true if this socket is ready to read the next message.
	 * @return True if this socket is ready, false otherwise
	 */
	public boolean readyToRead() throws SevereException
	{
		try {
			return in.available() > 0;
		} catch (IOException e) {
			throw new SevereException(e);
		}
	}
	
	/**
	 * Closes the client
	 */
	public void closeClient() throws SevereException
	{
		try 
		{
			in.close();
			out.close();
			server.close();
		} catch (IOException e) 
		{
			throw new SevereException(e);
		}
	}

	/**
	 * Writes a text message to the server
	 * @param message The message to send
	 * @throws IOException
	 */
	public void sendTextMessage(String message) throws IOException
	{
		TextMessage msg = new TextMessage(id);
		msg.setText(message);
		out.writeObject(msg);
	}
	

	/**
	 * Sends a message to the server
	 * @param message The message to send
	 * @throws IOException
	 */
	public void sendMessage(Message message) throws IOException
	{
		out.writeObject(message);
	}
	

	/**
	 * Writes an initialization message to the server
	 * @param data The data to send with the message
	 * @param type The type of the message
	 * @throws IOException 
	 */
	public void sendInitialMessage(String data, InitialMessageType type) throws IOException
	{
		InitialMessage initMessage = new InitialMessage(id, type);
		initMessage.setData(data);
		out.writeObject(initMessage);
	}
	
	/**
	 * Reads a message of a particular type 
	 * @param <E> The type of the message
	 * @param messageClass The class of the message to check for
	 * @return the message received if the message is of the specified class, null otherwise
	 * @throws IOException
	 * @throws SevereException
	 */
	public <E extends Message> E readMessage(Class messageClass) throws IOException, SevereException
	{
		Message msg = readMessage();
		if (msg == null) return null;
		if (messageClass.isInstance(msg))
		{
			return (E)msg;
		}
		else
		{
			return null;
		}
	}
	
	/**
	 * Reads the next message from the client. Will throw a SevereException if the data
	 * received is not a Message type
	 * @return The next message object
	 * @throws IOException on i/o error with the socket
	 * @throws SevereException if data received is not of Message type
	 */
	public Message readMessage() throws IOException, SevereException
	{
		try
		{
			Object o = in.readObject();
			if (!(o instanceof Message))
			{
				throw new SevereException ("Object received not an instance of Message");
			}
			Message msg = (Message)o;
			return msg;
		}
		catch (EOFException e)
		{
			return null;
		}
		catch (ClassNotFoundException e)
		{
			
			throw new SevereException(e.getMessage());
		}
	}

	/**
	 * @return true if this client is running
	 */
	public synchronized boolean isRunning() {
		return running;
	}

	/**
	 * Starts the client
	 */
	public synchronized void startClient()
	{
		running = true;
	}

	/**
	 * Stops the client
	 */
	public synchronized void stopClient()
	{
		running = false;
	}


	public boolean isConnected() {
		return connected;
	}
	
	
	
}
