package coloreti.network.server;

import java.io.*;
import java.net.*;
import java.util.logging.Logger;

import coloreti.network.exceptions.SevereException;
import coloreti.network.messages.InitialMessage;
import coloreti.network.messages.Message;
import coloreti.network.messages.TextMessage;
import coloreti.network.messages.InitialMessage.InitialMessageType;

/**
 * This class implements a client handling thread. The server maintains one
 * thread for each client connected to it.
 * @author ctheng
 *
 */
public class ClientThread implements Runnable {

	private int clientId;
	private ObjectInputStream in;
	private ObjectOutputStream out;
	private Socket clientSocket;
	private ServerBase server;
	private String clientName;
	private Message lastMessageReceived;
	private Message lastMessageSent;
	private boolean running;
	private boolean ready;
	
	private Logger LOGGER = Logger.getLogger("tta.server.ClientThread");

	public ClientThread(Socket client, ServerBase server)
	{
		this.clientSocket = client;
		this.server = server;
		running = false;
		ready = false;
	}
	
	/**
	 * Gets the id of this client
	 * @return The id
	 */
	public int getClientId()
	{
		return clientId;
	}
	
	/**
	 * Sets the id of this client
	 * @param id The client id
	 */
	public void setClientId(int id)
	{
		clientId = id;
	}
	
	/**
	 * Sets the name of the client
	 * @param clientName the clientName to set
	 */
	public void setClientName(String clientName) 
	{
		this.clientName = clientName;
	}

	/**
	 * Gets the name of the client
	 * @return the clientName
	 */
	public String getClientName() 
	{
		return clientName;
	}

	public void run()
	{
		
		try 
		{
			String line;
			in = new ObjectInputStream(clientSocket.getInputStream());
			out = new ObjectOutputStream(clientSocket.getOutputStream());
			out.flush();
			/* Receive the client's name from the client */
			InitialMessage nameMsg = readMessage(InitialMessage.class);
			if (nameMsg == null || nameMsg.getType() != InitialMessageType.NAME_SEND)
			{
				throw new SevereException("Unknown message received from client.");
			}
			
			/* It is important to set the client's name before registering the client
			 * Because the server will send the client's name to all other clients */
			clientName = nameMsg.getData();
			clientId = server.registerClient(this);
			
			/* Send the client a list of client's already connected */
			InitialMessage welcomeMsg = new InitialMessage(clientId, InitialMessageType.WELCOME);
			welcomeMsg.setData(server.getClientNameList());
			sendMessage(welcomeMsg);
			startClient();
			// loop and receive input
			while (isRunning())
			{
				Message msg = readMessage();
				server.messageReceived(clientId, msg);
			}
			
			sendTextMessage("Goodbye, " + getClientName());
		}
		catch (IOException e)
		{
			LOGGER.severe(e.getMessage());			
		}
		catch (SevereException e)
		{
			LOGGER.severe(e.getMessage());
		}
		finally
		{
			try 
			{
				server.removeClient(clientId);
				in.close();
				out.close();
				clientSocket.close();	
			}
			catch (IOException e)
 			{
				LOGGER.severe("Error closing client " + clientId + " " + e.getMessage());
			}
		}
	}

	public Socket getSocket()
	{
		return clientSocket;
	}

	/**
	 * Writes a message to the client from the server
	 * @param msg The message to send
	 */
	public void sendTextMessage(String msg) throws IOException
	{
		sendTextMessage(ServerBase.RESERVED_SERVER_ID, msg);
	}
	
	/**
	 * Writes a message to the client
	 * @param id The id of the sender
	 * @param msg The message to send
	 */
	public void sendTextMessage(int id, String msg) throws IOException
	{
		TextMessage textMessage = new TextMessage(id);
		textMessage.setText(msg);
		out.writeObject(textMessage);
		lastMessageSent = textMessage;
	}
	
	/**
	 * Writes an initialization message to the client from the server
	 * @param data The data in the message
	 * @param type The type of the message
	 * @throws IOException
	 */
	public void sendInitialMessage(String data, InitialMessageType type) throws IOException
	{
		sendInitialMessage(ServerBase.RESERVED_SERVER_ID, data, type);
	}
	
	/**
	 * Writes an initialization message to the client
	 * @param id The id of the sender
	 * @param data The data to send with the message
	 * @param type The type of the message
	 * @throws IOException 
	 */
	public void sendInitialMessage(int id, String data, InitialMessageType type) throws IOException
	{
		server.LOGGER.info("Sending initial message " + type.getInfo());
		InitialMessage initMessage = new InitialMessage(id, type);
		initMessage.setData(data);
		out.writeObject(initMessage);
		lastMessageSent = initMessage;
	}
	
	/**
	 * Reads a message of a particular type 
	 * @param <E> The type of the message
	 * @param messageClass The class of the message to check for
	 * @return the message received if the message is of the specified class, null otherwise
	 * @throws IOException
	 * @throws SevereException
	 */
	public <E extends Message> E readMessage(Class messageClass) throws IOException, SevereException
	{
		Message msg = readMessage();
		if (msg == null) return null;
		lastMessageReceived = msg;
		if (messageClass.isInstance(msg))
		{
			return (E)msg;
		}
		else
		{
			return null;
		}
	}
	
	/**
	 * Sends a message to the client.
	 * @param message The message to send
	 * @throws IOException
	 */
	public void sendMessage(Message message) throws IOException
	{
		out.writeObject(message);
	}
	
	/**
	 * Reads the next message from the client. Will throw a SevereException if the data
	 * received is not a Message type
	 * @return The next message object
	 * @throws IOException on i/o error with the socket
	 * @throws SevereException if data received is not of Message type
	 */
	public Message readMessage() throws IOException, SevereException
	{
		try
		{
			Object o = in.readObject();
			if (!(o instanceof Message))
			{
				throw new SevereException ("Object received not an instance of Message");
			}
			Message msg = (Message)o;
			if (msg.getSenderId() != clientId && msg.getSenderId() != -1)
			{
				throw new SevereException ("Message received with unknown sender id");
			}
			return msg;
		}
		catch (ClassNotFoundException e)
		{
			
			throw new SevereException(e.getMessage());
		}
	}
	
	/**
	 * @return true if this client is running
	 */
	public synchronized boolean isRunning() {
		return running;
	}

	/**
	 * Starts the client
	 */
	public synchronized void startClient()
	{
		running = true;
	}

	/**
	 * Stops the client
	 */
	public synchronized void stopClient()
	{
		running = false;
	}
	
	/**
	 * Sets if the client is ready
	 * @param ready Set to true if the client is ready
	 */
	public void setReady(boolean ready)
	{
		this.ready = ready;
	}
	
	/**
	 * Checks if the client is ready
	 * @return true if the client is ready
	 */
	public boolean isReady()
	{
		return ready;
	}

	
	@Override
	public boolean equals(Object o)
	{
		if (o == this) return true;
		if (!(o instanceof ClientThread)) return false;
		ClientThread ct = (ClientThread)o;
		return ((this.clientSocket == ct.clientSocket) && (this.clientName == ct.clientName));
	}
}
