package coloreti.network.server;

import java.util.*;
import java.util.logging.*;
import java.io.*;
import java.net.*;

import coloreti.GameEngine;
import coloreti.network.exceptions.SevereException;
import coloreti.network.messages.*;
import coloreti.network.messages.InitialMessage.InitialMessageType;

public class ServerBase 
{
	public enum ServerGameState
	{
		GAME_LOBBY,
		GAME_STARTED,
		GAME_OVER
	};
	
	public static final int RESERVED_SERVER_ID = 1048576;
	public static final String SERVER_NAME = "color-server";
	public static final int MAX_PLAYERS = 6;
	
	private int port;
	private ServerSocket serverSocket;
	protected HashMap<Integer, ClientThread> clients;
	private Random rng;
	private ServerGameState gameState;
	private long currentTime;
	private GameEngine engine;
	
	Logger LOGGER = Logger.getLogger("tta.server.ServerBase");
	
	/**
	 * Creates a new listening server on the specified port
	 * @param port The port to listen on
	 */
	public ServerBase(int port)
	{
		this.port = port;
		rng = new Random();
		gameState = ServerGameState.GAME_LOBBY;
		clients = new HashMap<Integer, ClientThread>();
		engine = null;
	}
	
	public void initialize() throws Exception
	{
		try
		{
			serverSocket = new ServerSocket(port);
		}
		catch (IOException e)
		{
			e.printStackTrace();
			LOGGER.severe("Unable to open listening socket on port " + port);
			throw new SevereException ("Unable to open listening socket.");
		}
		LOGGER.info("Listening server started on port " + port);
	}
	
	public void run() throws Exception
	{
		LOGGER.info("Running");
		while (true)
		{
			/* Loop as long as the server is still at the lobby. Once the game has started, refuse
			 * additional connections
			 */
			if (!gameState.equals(ServerGameState.GAME_LOBBY))
			{
				break;
			}
			Socket client = null;
			try 
			{
				client = serverSocket.accept();
				if (client == null) 
				{
					continue;
				}
				synchronized(clients)
				{
					if (clients.size() < MAX_PLAYERS)
					{
						ClientThread thread = new ClientThread(client, this);
						new Thread(thread).start();
						LOGGER.info("Started client thread to client with IP " + client.getInetAddress());
					}
					else
					{
						client.close();
					}
				}
			}
			catch (IOException e)
			{
				continue;
			}
			
		}
		
		/* The game is now started */
		currentTime = System.currentTimeMillis();
		long accumTime = 0;
		engine.startGame();
		while (true)
		{
			long elapsedTime = System.currentTimeMillis() - currentTime;
			currentTime += elapsedTime;
			accumTime += elapsedTime;
			long time = 0;
			if (accumTime > 20)
			{
				time += accumTime;
				accumTime -= 20;
			}
			if (time > 0)
			{
				engine.update(time);
			}
			
		}
	}
	
	/**
	 * Gets a list of clients connected to the server, delimited by ; 
	 * @return A list of clients
	 */
	public String getClientNameList()
	{
		String nameList = "";
		synchronized(clients)
		{
			for (ClientThread client : clients.values())
			{
				nameList += client.getClientName() + ";";
			}
		}
		return nameList;
	}
	
	/**
	 * Sends a message to all clients
	 * @param message The message to send
	 * @throws IOException
	 */
	public void serverMessage(Message message) throws IOException
	{
		//LOGGER.info("serverMessage-" + message);
		synchronized (clients)
		{
			for (ClientThread client: clients.values())
			{
				client.sendMessage(message);
			}
		}
	}

	/**
	 * Sends a text message to all clients
	 * @param message The message to send
	 * @throws IOException 
	 */
	public void serverTextMessage(String message) throws IOException
	{
		LOGGER.info("serverTextMessage-" + message);
		synchronized (clients)
		{
			for (ClientThread client : clients.values())
			{
				client.sendTextMessage("SERVER: " + message);
			}
		}
	}
	
	/**
	 * Sends a message to a single client
	 * @param clientId The id of the client to send to
	 * @param message The message to send
	 * @throws IOException 
	 */
	public void serverTextMessage(int clientId, String message) throws IOException
	{
		LOGGER.info("serverMessage-" + clientId + ": " + message);
		ClientThread client = clients.get(clientId);
		if (client != null)
		{
			client.sendTextMessage("SERVER: " + message);
		}
		else
		{
			LOGGER.info("WARNING: Tried to send message, but client with id " + clientId + " not found.");
		}
	}
	
	
	/**
	 * Sends a message from one client to another
	 * @param senderId The id of the sending client thread
	 * @param receiverId The id of the receiving client thread
	 * @param message The message to send
	 * @throws IOException 
	 */
	public void privateTextMessage(int senderId, int receiverId, String message) throws IOException
	{
		LOGGER.info("privateMessage-" + senderId + "->" + receiverId + ": " + message);
		ClientThread sender = clients.get(senderId);
		ClientThread receiver = clients.get(receiverId);
		
		if (sender == null || receiver == null)
		{
			LOGGER.warning("Invalid ID for sender or receiver client");
			return;
		}
		
		String senderName = sender.getClientName();
		receiver.sendTextMessage(senderId, senderName + ": " + message);
	}
	
	/**
	 * Echoes message to all connected clients except the sending client
	 * @param clientId The client that sent the message
	 * @param message The message to send
	 * @throws IOException 
	 */
	public void echoToAll(int clientId, String message) throws IOException
	{
		LOGGER.info("echoToAll-" + clientId + ": " + message);
		ClientThread sendingClient = clients.get(clientId);
		String clientName = "";
		if (sendingClient == null)
		{
			LOGGER.warning("Message received from unknown client");
			return;
		}
		else
		{
			clientName = sendingClient.getClientName();
		}
		synchronized (clients)
		{
			for (ClientThread client : clients.values())
			{
				client.sendTextMessage(clientId, clientName + ": " + message);
			}
		}
	}
	
	/**
	 * Registers a new client thread
	 * @param client The client thread to register
	 * @return The id of the new client
	 * @throws IOException 
	 */
	public int registerClient(ClientThread client) throws IOException
	{
		synchronized(clients)
		{
			int newId = rng.nextInt();
			while (clients.containsKey(newId) || newId == RESERVED_SERVER_ID)
			{
				newId = rng.nextInt();
			}
			LOGGER.info("Registered new client " + client.getSocket().getInetAddress() + " with id " + newId);
			// send a client joined message to all clients
			InitialMessage imsg = new InitialMessage(RESERVED_SERVER_ID, InitialMessageType.CLIENT_JOINED);
			imsg.setData(client.getClientName());
			serverMessage(imsg);
			clients.put(newId, client);
			
			return newId;
		}
	}
	
	/**
	 * Removes a client from this server
	 * @param id The id to remove
	 * @throws IOException 
	 */
	public void removeClient(int id) throws IOException
	{
		synchronized(clients)
		{
			ClientThread client = clients.get(id);
			if (client == null) return;
			clients.remove(id);
			LOGGER.info("Removed client with id " + id);
			InitialMessage msg = new InitialMessage(RESERVED_SERVER_ID, InitialMessageType.CLIENT_LEFT);
			msg.setData(client.getClientName());
			serverMessage(msg);
		}
	}

	/**
	 * Handles a message received from a client thread
	 * @param clientId the id of the sender
	 * @param msg The message received
	 * @throws IOException 
	 */
	public void messageReceived(int clientId, Message msg) throws IOException {
		if (gameState.equals(ServerGameState.GAME_LOBBY) && (msg instanceof InitialMessage))
		{
			InitialMessage imsg = (InitialMessage)msg;
			if (imsg.getType().equals(InitialMessageType.READY))
			{
				/* Sets if this client is ready */
				ClientThread client = clients.get(clientId);
				client.setReady(true);
				client.sendTextMessage("Received READY message.");
				LOGGER.info(client.getClientName() + " has sent the READY message.");
				boolean allReady = true;
				for (ClientThread cli : clients.values())
				{
					if (!cli.isReady())
					{
						allReady = false;
						break;
					}
				}
				
				if (allReady)
				{
					// start game
					if (clients.size() >= 2)
					{
						startGame();
					}
					else
					{
						serverTextMessage("At least 2 players needed to start the game.");
					}
				}
			}
		}
		else if ((msg instanceof TextMessage))
		{
			/* Always echo text messages to all other clients, regardless of what state */
			String text = ((TextMessage)msg).getText();
			if (text.equals("/quit")) 
			{
				ClientThread client = clients.get(clientId);
				client.stopClient();
				return;
			}
			echoToAll(clientId, text);
		}
		else if ((msg instanceof GameActionMessage))
		{
			if (!gameState.equals(ServerGameState.GAME_STARTED))
			{
				return;
			}
			if (msg.getSenderId() != clientId)
			{
				return;
			}
			// server has received an action message from a client
			// simply forward it to the engine
			engine.applyAction((GameActionMessage)msg);
			serverSocket.close();
		}
	}
	
	/**
	 * This method is called once all players are ready.
	 * @throws IOException 
	 */
	private void startGame() throws IOException
	{
		serverTextMessage("Starting game in 5 seconds...");
		try {
			Thread.sleep(5000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		synchronized(clients)
		{
			engine = new GameEngine(this);
			engine.generateNewGame(clients.size());
			// generate the game start message
			GameStartMessage gsm = engine.getGameStartMessage(RESERVED_SERVER_ID);
			// reassign ids to each client randomly
			ArrayList<ClientThread> clientThreads = new ArrayList<ClientThread>();
			clientThreads.addAll(clients.values());
			
			Collections.shuffle(clientThreads);
			clients.clear();
			for (int i = 0; i < clientThreads.size(); i++)
			{
				clientThreads.get(i).setClientId(i);
				clients.put(i, clientThreads.get(i));
			}
			for (int i = 0; i < clientThreads.size(); i++)
			{
				String name = clients.get(i).getClientName();
				gsm.getPlayerData().get(i).setName(name);
			}
			// send the game start message to each client
			for (int i = 0; i < clients.size(); i++)
			{
				ClientThread client = clients.get(i);
				gsm.setReceiverId(i);
				LOGGER.info("Sending GameStartMessage to player " + i + ": " + client.getClientName());
				client.sendMessage(gsm);
			}
		}
		
		gameState = ServerGameState.GAME_STARTED;
	}
	
	/**
	 * Gets the single random number generator
	 * @return The random number generator for this server
	 */
	public Random getRandom()
	{
		return rng;
	}
}
